from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin

from sqlalchemy_utils import UUIDType


class HardwareItemDepartmentLink(db.Model, BaseMixin, SerializerMixin):
    """
    Uses a many-to-many relationship to link hardware items with departments.
    It can be used to track which hardware items are used by which departments in
    order to set budget forecasting.
    """

    __tablename__ = "hardware_item_department_link"
    department_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("department.id"),
        primary_key=True,
        index=True,
    )
    hardware_item_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("hardware_item.id"),
        primary_key=True,
        index=True,
    )

    __table_args__ = (
        db.UniqueConstraint(
            "hardware_item_id",
            "department_id",
            name="hardware_item_department_link_uc",
        ),
    )


class SoftwareDepartmentLink(db.Model, BaseMixin, SerializerMixin):
    """
    Uses a many-to-many relationship to link software with departments.
    It can be used to track which software is used by which departments in
    order to set budget forecasting.
    """

    __tablename__ = "software_department_link"
    department_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("department.id"),
        primary_key=True,
        index=True,
    )
    software_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("software.id"),
        primary_key=True,
        index=True,
    )

    __table_args__ = (
        db.UniqueConstraint(
            "department_id",
            "software_id",
            name="software_department_link_uc",
        ),
    )


class Department(db.Model, BaseMixin, SerializerMixin):
    """
    Studio department like modeling, animation, etc.
    """

    name = db.Column(db.String(80), unique=True, nullable=False)
    color = db.Column(db.String(7), nullable=False)
    archived = db.Column(db.Boolean(), default=False)



"""modify salary scale model

Revision ID: d25118cddcaa
Revises: 7a16258f2fab
Create Date: 2025-04-23 22:49:26.645851

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils


# revision identifiers, used by Alembic.
revision = "d25118cddcaa"
down_revision = "7a16258f2fab"
branch_labels = None
depends_on = None

from zou.app.models.person import POSITION_TYPES, SENIORITY_TYPES


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("salary_scale", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "position",
                sqlalchemy_utils.types.choice.ChoiceType(POSITION_TYPES),
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "seniority",
                sqlalchemy_utils.types.choice.ChoiceType(SENIORITY_TYPES),
                nullable=False,
            )
        )
        batch_op.add_column(sa.Column("salary", sa.Integer(), nullable=False))
        batch_op.drop_column("mid_artist_salary")
        batch_op.drop_column("junior_lead_salary")
        batch_op.drop_column("mid_lead_salary")
        batch_op.drop_column("mid_supervisor_salary")
        batch_op.drop_column("junior_supervisor_salary")
        batch_op.drop_column("senior_artist_salary")
        batch_op.drop_column("senior_lead_salary")
        batch_op.drop_column("junior_artist_salary")
        batch_op.drop_column("senior_supervisor_salary")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("salary_scale", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "senior_supervisor_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "junior_artist_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "senior_lead_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "senior_artist_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "junior_supervisor_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "mid_supervisor_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "mid_lead_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "junior_lead_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.add_column(
            sa.Column(
                "mid_artist_salary",
                sa.INTEGER(),
                autoincrement=False,
                nullable=False,
            )
        )
        batch_op.drop_column("salary")
        batch_op.drop_column("seniority")
        batch_op.drop_column("position")

    # ### end Alembic commands ###



from datetime import date, timedelta
from sqlalchemy.exc import StatementError
from sqlalchemy.dialects.postgresql import insert
from sqlalchemy import update


from zou.app.models.entity import Entity
from zou.app.models.entity_type import EntityType
from zou.app.models.milestone import Milestone
from zou.app.models.schedule_item import ScheduleItem
from zou.app.models.task import Task
from zou.app.models.task_type import TaskType
from zou.app.models.production_schedule_version import (
    ProductionScheduleVersion,
    ProductionScheduleVersionTaskLink,
)
from zou.app.utils import events, fields, cache
from zou.app.services import (
    assets_service,
    base_service,
    shots_service,
    tasks_service,
    projects_service,
)
from zou.app import db

from zou.app.services.exception import (
    ProductionScheduleVersionNotFoundException,
)


def clear_production_schedule_version_cache(production_schedule_version_id):
    cache.cache.delete_memoized(
        get_production_schedule_version, production_schedule_version_id
    )
    cache.cache.delete_memoized(
        get_production_schedule_version, production_schedule_version_id, True
    )


def get_schedule_items(project_id):
    """
    Get all project schedule items (mainly for sync purpose).
    """
    schedule_items = ScheduleItem.query.filter_by(project_id=project_id).all()
    return fields.serialize_list(schedule_items)


def get_task_types_schedule_items(project_id):
    """
    Return all schedule items for given project. If no schedule item exists
    for a given task type, it creates one.
    """
    task_types = tasks_service.get_task_types_for_project(project_id)
    task_types = [
        task_type
        for task_type in task_types
        if task_type["for_entity"] in ["Asset", "Shot"]
    ]
    task_type_map = base_service.get_model_map_from_array(task_types)
    schedule_items = set(
        ScheduleItem.query.filter_by(project_id=project_id)
        .filter(ScheduleItem.object_id == None)
        .all()
    )
    schedule_item_map = {
        str(schedule_item.task_type_id): schedule_item
        for schedule_item in schedule_items
    }

    new_schedule_items = set()
    schedule_item_to_remove = set()
    for schedule_item in schedule_items:
        if schedule_item.task_type_id is not None:
            if str(schedule_item.task_type_id) not in task_type_map:
                schedule_item_to_remove.add(schedule_item)

    for task_type in task_types:
        if task_type["id"] not in schedule_item_map:
            new_schedule_item = ScheduleItem.create(
                project_id=project_id,
                start_date=date.today(),
                end_date=date.today() + timedelta(days=1),
                task_type_id=task_type["id"],
            )
            new_schedule_items.add(new_schedule_item)
            events.emit(
                "schedule-item:new",
                {"schedule_item_id": str(new_schedule_item.id)},
                project_id=project_id,
            )

    schedule_items = (
        schedule_items.union(new_schedule_items) - schedule_item_to_remove
    )
    return sorted(
        [schedule_item.present() for schedule_item in schedule_items],
        key=lambda x: x["start_date"],
    )


def get_asset_types_schedule_items(project_id, task_type_id):
    """
    Return all asset type schedule items for given project. If no schedule item
    exists for a given asset type, it creates one.
    """
    asset_types = assets_service.get_asset_types_for_project(project_id)
    asset_type_map = base_service.get_model_map_from_array(asset_types)
    existing_schedule_items = set(
        ScheduleItem.query.join(
            EntityType, ScheduleItem.object_id == EntityType.id
        )
        .filter(ScheduleItem.project_id == project_id)
        .filter(ScheduleItem.task_type_id == task_type_id)
        .all()
    )
    return get_entity_schedule_items(
        project_id,
        task_type_id,
        asset_types,
        asset_type_map,
        existing_schedule_items,
    )


def get_episodes_schedule_items(project_id, task_type_id):
    """
    Return all episode schedule items for given project. If no schedule item
    exists for a given asset type, it creates one.
    """
    episode_type = shots_service.get_episode_type()
    episodes = shots_service.get_episodes_for_project(project_id)
    episodes_map = base_service.get_model_map_from_array(episodes)
    existing_schedule_items = set(
        ScheduleItem.query.join(Entity, ScheduleItem.object_id == Entity.id)
        .filter(ScheduleItem.project_id == project_id)
        .filter(Entity.entity_type_id == episode_type["id"])
        .filter(ScheduleItem.task_type_id == task_type_id)
        .all()
    )
    return get_entity_schedule_items(
        project_id,
        task_type_id,
        episodes,
        episodes_map,
        existing_schedule_items,
    )


def get_sequences_schedule_items(project_id, task_type_id, episode_id=None):
    """
    Return all asset type schedule items for given project. If no schedule item
    exists for a given asset type, it creates one.
    """
    if episode_id is not None:
        sequences = shots_service.get_sequences_for_episode(episode_id)
    else:
        sequences = shots_service.get_sequences_for_project(project_id)
    sequence_map = base_service.get_model_map_from_array(sequences)
    sequence_type = shots_service.get_sequence_type()

    query = (
        ScheduleItem.query.join(Entity, ScheduleItem.object_id == Entity.id)
        .filter(ScheduleItem.project_id == project_id)
        .filter(Entity.entity_type_id == sequence_type["id"])
        .filter(ScheduleItem.task_type_id == task_type_id)
    )
    if episode_id is not None:
        query = query.filter(Entity.parent_id == episode_id)
    existing_schedule_items = set(query.all())

    return get_entity_schedule_items(
        project_id,
        task_type_id,
        sequences,
        sequence_map,
        existing_schedule_items,
    )


def get_entity_schedule_items(
    project_id, task_type_id, to_create, to_create_map, existing_schedule_items
):
    schedule_item_map = {
        str(schedule_item.object_id): schedule_item
        for schedule_item in existing_schedule_items
    }

    new_schedule_items = set()
    schedule_item_to_remove = set()
    for schedule_item in existing_schedule_items:
        if schedule_item.object_id is not None:
            if str(schedule_item.object_id) not in to_create_map:
                schedule_item_to_remove.add(schedule_item)

    for entity in to_create:
        if entity["id"] not in schedule_item_map:
            new_schedule_item = ScheduleItem.create(
                project_id=project_id,
                start_date=date.today(),
                end_date=date.today() + timedelta(days=1),
                object_id=entity["id"],
                task_type_id=task_type_id,
            )
            events.emit(
                "schedule-item:new",
                {"schedule_item_id": str(new_schedule_item.id)},
                project_id=project_id,
            )
            new_schedule_items.add(new_schedule_item)

    schedule_items = (
        existing_schedule_items.union(new_schedule_items)
        - schedule_item_to_remove
    )

    results = []
    for schedule_item in schedule_items:
        result = schedule_item.present()
        result["name"] = to_create_map[result["object_id"]]["name"]
        results.append(result)

    return sorted(results, key=lambda x: x["name"])


def get_milestones_for_project(project_id):
    """
    Return all milestones related to given project.
    """
    query = Milestone.query.filter_by(project_id=project_id)
    return [milestone.present() for milestone in query.all()]


def get_production_schedule_version_raw(production_schedule_version_id):
    """
    Get production schedule version matching given id.
    """
    try:
        production_schedule_version = ProductionScheduleVersion.get(
            production_schedule_version_id
        )
    except StatementError:
        raise ProductionScheduleVersionNotFoundException

    if production_schedule_version is None:
        raise ProductionScheduleVersionNotFoundException

    return production_schedule_version


@cache.memoize_function(120)
def get_production_schedule_version(
    production_schedule_version_id, relations=False
):
    """
    Get production schedule version matching given id and serialize it.
    """
    return get_production_schedule_version_raw(
        production_schedule_version_id
    ).serialize(relations=relations)


def get_production_schedule_version_task_links(
    production_schedule_version_id, task_type_id=None, relations=False
):
    """
    Get all task links for given production schedule version.
    """
    query = ProductionScheduleVersionTaskLink.query.filter_by(
        production_schedule_version_id=production_schedule_version_id
    )

    if task_type_id is not None:
        query = (
            query.join(Task)
            .join(TaskType)
            .filter(Task.task_type_id == task_type_id)
        )

    return fields.serialize_models(query.all(), relations=relations)


def update_production_schedule_version(production_schedule_version_id, data):
    """
    Update production schedule version with given id with data.
    """
    production_schedule_version = get_production_schedule_version_raw(
        production_schedule_version_id
    )

    production_schedule_version.update(data)
    clear_production_schedule_version_cache(production_schedule_version_id)
    events.emit(
        "production_schedule_version:update",
        {"production_schedule_version_id": production_schedule_version_id},
        project_id=str(production_schedule_version.project_id),
    )
    return production_schedule_version.serialize()


def set_production_schedule_version_task_links_from_production(
    production_schedule_version_id,
):
    """
    Set task links for given production schedule version from tasks in the
    production.
    """
    production_schedule_version = get_production_schedule_version(
        production_schedule_version_id
    )

    tasks = (
        db.session.query(Task)
        .filter(Task.project_id == production_schedule_version["project_id"])
        .all()
    )

    rows = [
        {
            "id": fields.gen_uuid(),
            "production_schedule_version_id": production_schedule_version_id,
            "task_id": task.id,
            "start_date": task.start_date,
            "due_date": task.due_date,
            "estimation": task.estimation,
        }
        for task in tasks
    ]
    insert_stmt = insert(ProductionScheduleVersionTaskLink).values(rows)
    insert_stmt = insert_stmt.on_conflict_do_update(
        index_elements=["production_schedule_version_id", "task_id"],
        set_={
            "start_date": insert_stmt.excluded.start_date,
            "due_date": insert_stmt.excluded.due_date,
            "estimation": insert_stmt.excluded.estimation,
        },
    ).returning(ProductionScheduleVersionTaskLink)

    results = db.session.execute(insert_stmt).scalars().all()

    tasks_map = {task.id: task for task in tasks}

    for task_link in results:
        task_link.assignees = tasks_map[task_link.task_id].assignees

    db.session.commit()

    return fields.serialize_models(results, relations=True)


def set_production_schedule_version_task_links_from_production_schedule_version(
    production_schedule_version_id, other_production_schedule_version_id
):
    """
    Set task links for given production schedule version from another.
    """

    other_links = (
        db.session.query(ProductionScheduleVersionTaskLink)
        .filter(
            ProductionScheduleVersionTaskLink.production_schedule_version_id
            == other_production_schedule_version_id
        )
        .all()
    )

    rows = [
        {
            "id": fields.gen_uuid(),
            "production_schedule_version_id": production_schedule_version_id,
            "task_id": links.task_id,
            "start_date": links.start_date,
            "due_date": links.due_date,
            "estimation": links.estimation,
        }
        for links in other_links
    ]

    insert_stmt = insert(ProductionScheduleVersionTaskLink).values(rows)
    insert_stmt = insert_stmt.on_conflict_do_update(
        index_elements=["production_schedule_version_id", "task_id"],
        set_={
            "start_date": insert_stmt.excluded.start_date,
            "due_date": insert_stmt.excluded.due_date,
            "estimation": insert_stmt.excluded.estimation,
        },
    ).returning(ProductionScheduleVersionTaskLink)

    results = db.session.execute(insert_stmt).scalars().all()

    tasks_map = {link.task_id: link for link in other_links}

    for task_link in results:
        task_link.assignees = tasks_map[task_link.task_id].assignees

    db.session.commit()

    update_production_schedule_version(
        production_schedule_version_id,
        {"production_schedule_from": other_production_schedule_version_id},
    )

    return fields.serialize_models(results, relations=True)


def apply_production_schedule_version_to_production(
    production_schedule_version_id,
):
    """
    Apply production schedule version to production.
    """

    stmt = (
        update(Task)
        .values(
            start_date=ProductionScheduleVersionTaskLink.start_date,
            due_date=ProductionScheduleVersionTaskLink.due_date,
            estimation=ProductionScheduleVersionTaskLink.estimation,
        )
        .where(Task.id == ProductionScheduleVersionTaskLink.task_id)
        .where(
            ProductionScheduleVersionTaskLink.production_schedule_version_id
            == production_schedule_version_id
        )
        .returning(Task)
    )

    results = db.session.execute(stmt).scalars().all()

    db.session.commit()

    for task in results:
        events.emit(
            "task:update",
            {"task_id": str(task.id)},
            project_id=str(task.project_id),
        )

    production_schedule_version = update_production_schedule_version(
        production_schedule_version_id, {"locked": True}
    )

    projects_service.update_project(
        production_schedule_version["project_id"],
        {"from_schedule_version_id": production_schedule_version_id},
    )

    return fields.serialize_models(results, relations=True)



from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin


class HardwareItem(db.Model, BaseMixin, SerializerMixin):
    """
    Describes hardware items available in the studio.
    """

    name = db.Column(db.String(40), unique=True, nullable=False)
    short_name = db.Column(db.String(20), nullable=False)
    archived = db.Column(db.Boolean, default=False)
    monthly_cost = db.Column(db.Integer, default=0)
    inventory_amount = db.Column(db.Integer, default=0)



from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin


class ProjectStatus(db.Model, BaseMixin, SerializerMixin):
    """
    Describes the state of the project (mainly open or closed).
    """

    name = db.Column(db.String(20), unique=True, nullable=False, index=True)
    color = db.Column(db.String(7), nullable=False)



from flask_restful import Resource

from flask_jwt_extended import jwt_required

from zou.app.services import (
    entities_service,
    news_service,
    preview_files_service,
    time_spents_service,
    user_service,
)


class EntityNewsResource(Resource):
    @jwt_required()
    def get(self, entity_id):
        """
        Retrieve all news linked to a given entity.
        ---
        tags:
          - Entities
        parameters:
          - in: path
            name: entity_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All news linked to given entity
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        return news_service.get_news_for_entity(entity_id)


class EntityPreviewFilesResource(Resource):
    @jwt_required()
    def get(self, entity_id):
        """
        Retrieve all preview files linked to a given entity.
        ---
        tags:
          - Entities
        parameters:
          - in: path
            name: entity_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All preview files linked to given entity
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        return preview_files_service.get_preview_files_for_entity(entity_id)


class EntityTimeSpentsResource(Resource):
    @jwt_required()
    def get(self, entity_id):
        """
        Retrieve all time spents linked to a given entity.
        ---
        tags:
          - Entities
        parameters:
          - in: path
            name: entity_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All time spents linked to given entity
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        return time_spents_service.get_time_spents_for_entity(entity_id)


class EntitiesLinkedWithTasksResource(Resource):
    @jwt_required()
    def get(self, entity_id):
        """
        Resource to retrieve the entities linked on a given entity.
        ---
        tags:
            - Entities
        parameters:
          - in: path
            name: entity_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Entities linked on given entity
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        user_service.check_entity_access(entity_id)
        return entities_service.get_linked_entities_with_tasks(entity_id)



from sqlalchemy_utils import UUIDType
from zou.app.utils import fields, date_helpers
import sqlalchemy as sa


class BaseMixin(object):
    id = sa.Column(
        UUIDType(binary=False), primary_key=True, default=fields.gen_uuid
    )

    # Audit fields
    created_at = sa.Column(
        sa.DateTime, default=date_helpers.get_utc_now_datetime
    )
    updated_at = sa.Column(
        sa.DateTime,
        default=date_helpers.get_utc_now_datetime,
        onupdate=date_helpers.get_utc_now_datetime,
    )



"""Add short_name for Asset Type entity_type

Revision ID: 971dbf5a0faf
Revises: a252a094e977
Create Date: 2024-06-20 19:51:15.758780

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "971dbf5a0faf"
down_revision = "a252a094e977"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "entity_type",
        sa.Column("short_name", sa.String(length=20), nullable=True),
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("entity_type", "short_name")
    # ### end Alembic commands ###



from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin


class FileStatus(db.Model, BaseMixin, SerializerMixin):
    """
    Describe the state of a given file.
    """

    name = db.Column(db.String(40), unique=True, nullable=False)
    color = db.Column(db.String(7), nullable=False)



from flask_restful import Resource
from flask_jwt_extended import jwt_required

from zou.app.mixin import ArgsMixin
from zou.app.utils import fields, permissions, date_helpers

from zou.app.services import events_service
from zou.app.services.exception import WrongParameterException


class EventsResource(Resource, ArgsMixin):
    @jwt_required()
    def get(self):
        """
        Retrieve last events.
        ---
        tags:
          - Events
        parameters:
          - in: query
            name: after
            type: string
            format: date
            example: "2022-07-12"
          - in: query
            name: before
            type: string
            format: date
            example: "2022-07-12"
          - in: query
            name: only_files
            type: boolean
            default: False
          - in: query
            name: limit
            type: integer
            default: 100
            example: 100
          - in: query
            name: project_id
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All login logs
        """
        args = self.get_args(
            [
                ("after", None, False),
                ("before", None, False),
                ("only_files", False, False),
                ("limit", 100, False),
                ("project_id", None, False),
                ("name", None, False),
            ],
        )

        permissions.check_manager_permissions()
        before = self.parse_date_parameter(args["before"])
        after = self.parse_date_parameter(args["after"])
        limit = args["limit"]
        only_files = args["only_files"] == "true"
        project_id = args.get("project_id", None)
        name = args["name"]
        if project_id is not None and not fields.is_valid_id(project_id):
            raise WrongParameterException(
                "The project_id parameter is not a valid id"
            )
        else:
            return events_service.get_last_events(
                after=after,
                before=before,
                limit=limit,
                only_files=only_files,
                project_id=project_id,
                name=name,
            )


class LoginLogsResource(Resource, ArgsMixin):
    @jwt_required()
    def get(self):
        """
        Retrieve all login logs.
        ---
        tags:
          - Events
        parameters:
          - in: query
            name: before
            type: string
            format: date
            example: "2022-07-12T00:00:00"
          - in: query
            name: limit
            type: integer
            example: 100
        responses:
            200:
                description: All login logs
        """
        args = self.get_args(["before", ("limit", 100)])

        permissions.check_manager_permissions()
        before = None
        if args["before"] is not None:
            before = date_helpers.get_datetime_from_string(args["before"])
        limit = args["limit"]
        return events_service.get_last_login_logs(before, limit)



"""add hardware table

Revision ID: 9af2df17a9d5
Revises: d5665dca188b
Create Date: 2025-07-14 16:30:41.472002

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "9af2df17a9d5"
down_revision = "d5665dca188b"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "hardware_item",
        sa.Column("name", sa.String(length=40), nullable=False),
        sa.Column("short_name", sa.String(length=20), nullable=False),
        sa.Column("archived", sa.Boolean(), nullable=True),
        sa.Column("monthly_cost", sa.Integer(), nullable=True),
        sa.Column("inventory_amount", sa.Integer(), nullable=True),
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=False,
        ),
        sa.Column("created_at", sa.DateTime(), nullable=True),
        sa.Column("updated_at", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("hardware_item")
    # ### end Alembic commands ###



from sqlalchemy_utils import UUIDType, ChoiceType

from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin
from zou.app.utils import fields

from sqlalchemy.dialects.postgresql import JSONB

STATUSES = [
    ("processing", "Processing"),
    ("ready", "Ready"),
    ("broken", "Broken"),
]

VALIDATION_STATUSES = [
    ("validated", "Validated"),
    ("rejected", "Rejected"),
    ("neutral", "Neutral"),
]


class PreviewFile(db.Model, BaseMixin, SerializerMixin):
    """
    Describes a file which is aimed at being reviewed. It is not a publication
    neither a working file.
    """

    name = db.Column(db.String(250))
    original_name = db.Column(db.String(250))
    revision = db.Column(db.Integer(), default=1)
    position = db.Column(db.Integer(), default=1)
    extension = db.Column(db.String(6))
    description = db.Column(db.Text())
    path = db.Column(db.String(400))
    source = db.Column(db.String(40))
    file_size = db.Column(db.BigInteger(), default=0)
    status = db.Column(
        ChoiceType(STATUSES), default="processing", nullable=False
    )
    validation_status = db.Column(
        ChoiceType(VALIDATION_STATUSES), default="neutral", nullable=False
    )
    annotations = db.Column(JSONB)
    width = db.Column(db.Integer(), default=0)
    height = db.Column(db.Integer(), default=0)
    duration = db.Column(db.Float, default=0)

    task_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("task.id"), index=True
    )
    person_id = db.Column(UUIDType(binary=False), db.ForeignKey("person.id"))
    source_file_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("output_file.id")
    )

    __table_args__ = (
        db.UniqueConstraint("name", "task_id", "revision", name="preview_uc"),
    )

    shotgun_id = db.Column(db.Integer, unique=True)

    is_movie = db.Column(db.Boolean, default=False)  # deprecated
    url = db.Column(db.String(600))  # deprecated
    uploaded_movie_url = db.Column(db.String(600))  # deprecated
    uploaded_movie_name = db.Column(db.String(150))  # deprecated

    def __repr__(self):
        return "<PreviewFile %s>" % self.id

    @classmethod
    def create_from_import(cls, data):
        del data["type"]
        if "comments" in data:
            del data["comments"]
        previous_data = cls.get(data["id"])
        if "status" not in data or data["status"] is None:
            data["status"] = "ready"
        if previous_data is None:
            return (cls.create(**data), False)
        else:
            previous_data.update(data)
            return (previous_data, True)

    def present(self):
        return fields.serialize_dict(
            {
                "id": self.id,
                "name": self.name,
                "original_name": self.original_name,
                "extension": self.extension,
                "revision": self.revision,
                "position": self.position,
                "file_size": self.file_size,
                "status": self.status,
                "validation_status": self.validation_status,
                "task_id": self.task_id,
                "person_id": self.person_id,
                "created_at": self.created_at,
            }
        )

    def present_minimal(self):
        return fields.serialize_dict(
            {
                "id": self.id,
                "name": self.name,
                "extension": self.extension,
                "revision": self.revision,
                "position": self.position,
            }
        )



import datetime

from flask_jwt_extended import jwt_required

from zou.app.models.person import (
    Person,
    ROLE_TYPES,
    CONTRACT_TYPES,
    TWO_FACTOR_AUTHENTICATION_TYPES,
)
from zou.app.services import (
    deletion_service,
    index_service,
    persons_service,
)
from zou.app.utils import permissions, auth, date_helpers

from zou.app.blueprints.crud.base import BaseModelsResource, BaseModelResource

from zou.app.mixin import ArgsMixin

from zou.app.services.exception import (
    WrongParameterException,
    PersonInProtectedAccounts,
)

from zou.app import config


class PersonsResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, Person)

    def all_entries(self, query=None, relations=False):
        if query is None:
            query = self.model.query

        if permissions.has_admin_permissions():
            if self.get_bool_parameter("with_pass_hash"):
                return [
                    person.serialize(relations=relations)
                    for person in query.all()
                ]
            else:
                return [
                    person.serialize_safe(relations=relations)
                    for person in query.all()
                ]
        else:
            return [
                person.present_minimal(relations=relations)
                for person in query.all()
            ]

    def check_read_permissions(self, options=None):
        return True

    def check_create_permissions(self, data):
        if (
            not data.get("is_bot", False)
            and data.get("active", True)
            and persons_service.is_user_limit_reached()
        ):
            raise WrongParameterException(
                "User limit reached.",
                {
                    "limit": config.USER_LIMIT,
                },
            )
        return permissions.check_admin_permissions()

    def check_creation_integrity(self, data):
        if "role" in data and data["role"] not in [
            role for role, _ in ROLE_TYPES
        ]:
            raise WrongParameterException("Invalid role")
        if "contract_type" in data and data["contract_type"] not in [
            contract_type for contract_type, _ in CONTRACT_TYPES
        ]:
            raise WrongParameterException("Invalid contract_type")
        if "two_factor_authentication" in data and data[
            "two_factor_authentication"
        ] not in [
            two_factor_authentication
            for two_factor_authentication, _ in TWO_FACTOR_AUTHENTICATION_TYPES
        ]:
            raise WrongParameterException("Invalid two_factor_authentication")

        if "expiration_date" in data and data["expiration_date"] is not None:
            try:
                if (
                    date_helpers.get_date_from_string(
                        data["expiration_date"]
                    ).date()
                    < datetime.date.today()
                ):
                    raise WrongParameterException(
                        "Expiration date can't be in the past."
                    )
            except WrongParameterException:
                raise
            except:
                raise WrongParameterException("Expiration date is not valid.")

        if "email" in data:
            try:
                data["email"] = auth.validate_email(data["email"])
            except auth.EmailNotValidException as e:
                raise WrongParameterException(str(e))

        return data

    def update_data(self, data):
        data = super().update_data(data)
        if "password" in data and data["password"] is not None:
            data["password"] = auth.encrypt_password(data["password"])
        if "email" in data:
            data["email"] = data["email"].strip()
        return data

    def post_creation(self, instance):
        instance_dict = instance.serialize(relations=True)
        if instance.is_bot:
            instance_dict["access_token"] = (
                persons_service.create_access_token_for_raw_person(instance)
            )
        if instance.active:
            index_service.index_person(instance)
        persons_service.clear_person_cache()
        return instance_dict


class PersonResource(BaseModelResource, ArgsMixin):
    def __init__(self):
        BaseModelResource.__init__(self, Person)
        self.protected_fields += ["password", "jti"]

    def check_read_permissions(self, instance):
        return True

    def check_update_permissions(self, instance_dict, data):
        if instance_dict["id"] != persons_service.get_current_user()["id"]:
            permissions.check_admin_permissions()
        return instance_dict

    def update_data(self, data, instance_id):
        data = super().update_data(data, instance_id)
        if not permissions.has_admin_permissions():
            if not permissions.has_person_permissions():
                data.pop("expiration_date", None)
            data.pop("role", None)
            data.pop("departments", None)
            data.pop("active", None)
            data.pop("is_bot", None)
            data.pop("archived", None)
            data.pop("login_failed_attemps", None)
            data.pop("last_login_failed", None)
            data.pop("is_generated_from_ldap", None)
            data.pop("ldap_uid", None)
            data.pop("last_presence", None)
            data.pop("studio_id", None)

        if "role" in data and data["role"] not in [
            role for role, _ in ROLE_TYPES
        ]:
            raise WrongParameterException("Invalid role")
        if "contract_type" in data and data["contract_type"] not in [
            contract_type for contract_type, _ in CONTRACT_TYPES
        ]:
            raise WrongParameterException("Invalid contract_type")
        if "two_factor_authentication" in data and data[
            "two_factor_authentication"
        ] not in [
            two_factor_authentication
            for two_factor_authentication, _ in TWO_FACTOR_AUTHENTICATION_TYPES
        ]:
            raise WrongParameterException("Invalid two_factor_authentication")

        if "expiration_date" in data and data["expiration_date"] is not None:
            try:
                if (
                    datetime.datetime.strptime(
                        data["expiration_date"], "%Y-%m-%d"
                    ).date()
                    < datetime.date.today()
                ):
                    raise WrongParameterException(
                        "Expiration date can't be in the past."
                    )
            except WrongParameterException:
                raise
            except:
                raise WrongParameterException("Expiration date is not valid.")

        if "email" in data:
            try:
                data["email"] = auth.validate_email(data["email"])
            except auth.EmailNotValidException as e:
                raise WrongParameterException(str(e))

        return data

    def check_delete_permissions(self, instance_dict):
        if instance_dict["id"] == persons_service.get_current_user()["id"]:
            raise permissions.PermissionDenied
        permissions.check_admin_permissions()
        return instance_dict

    def serialize_instance(self, instance, relations=True):
        if permissions.has_manager_permissions():
            return instance.serialize_safe(relations=relations)
        else:
            return instance.present_minimal(relations=relations)

    def pre_update(self, instance_dict, data):
        if (
            not instance_dict.get("active", False)
            and data.get("active", False)
            and not instance_dict.get("is_bot", False)
            and not data.get("is_bot", False)
            and persons_service.is_user_limit_reached()
        ):
            raise WrongParameterException("User limit reached.")
        if (
            instance_dict["email"] in config.PROTECTED_ACCOUNTS
            and instance_dict["id"] != persons_service.get_current_user()["id"]
        ):
            message = None
            if data.get("active") is False:
                message = "Can't set this person as inactive it's a protected account."
            elif data.get("role") is not None:
                message = "Can't change the role of this person it's a protected account."

            if message is not None:
                raise PersonInProtectedAccounts(message)
        return data

    def post_update(self, instance_dict, data):
        persons_service.clear_person_cache()
        index_service.remove_person_index(instance_dict["id"])
        person = persons_service.get_person_raw(instance_dict["id"])
        if person.active:
            index_service.index_person(person)
        instance_dict["departments"] = [
            str(department.id) for department in self.instance.departments
        ]
        if "expiration_date" in data:
            instance_dict["access_token"] = (
                persons_service.create_access_token_for_raw_person(
                    self.instance
                )
            )
        return instance_dict

    def post_delete(self, instance_dict):
        persons_service.clear_person_cache()
        return instance_dict

    @jwt_required()
    def delete(self, instance_id):
        """
        Delete a person corresponding at given ID and return it as a JSON
        object.
        """
        force = self.get_force()
        person = self.get_model_or_404(instance_id)
        person_dict = person.serialize()
        self.check_delete_permissions(person_dict)
        self.pre_delete(person_dict)
        deletion_service.remove_person(instance_id, force=force)
        index_service.remove_person_index(instance_id)
        self.emit_delete_event(person_dict)
        self.post_delete(person_dict)
        return "", 204



"""Change Comment.updated_by in Comment.editor_id

Revision ID: 307edd8c639d
Revises: addbad59c706
Create Date: 2025-03-10 12:53:19.482537

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import sqlalchemy_utils
import uuid

# revision identifiers, used by Alembic.
revision = "307edd8c639d"
down_revision = "addbad59c706"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("comment", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "editor_id",
                sqlalchemy_utils.types.uuid.UUIDType(binary=False),
                default=uuid.uuid4,
                nullable=True,
            )
        )
        batch_op.drop_index("ix_comment_updated_by")
        batch_op.create_index(
            batch_op.f("ix_comment_editor_id"), ["editor_id"], unique=False
        )
        batch_op.drop_constraint("comment_updated_by_fkey", type_="foreignkey")
        batch_op.create_foreign_key(None, "person", ["editor_id"], ["id"])
        batch_op.drop_column("updated_by")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("comment", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "updated_by", sa.UUID(), autoincrement=False, nullable=True
            )
        )
        batch_op.drop_constraint("comment_editor_id_fkey", type_="foreignkey")
        batch_op.create_foreign_key(
            "comment_updated_by_fkey", "person", ["updated_by"], ["id"]
        )
        batch_op.drop_index(batch_op.f("ix_comment_editor_id"))
        batch_op.create_index(
            "ix_comment_updated_by", ["updated_by"], unique=False
        )
        batch_op.drop_column("editor_id")

    # ### end Alembic commands ###



from sqlalchemy_utils import UUIDType, ChoiceType

from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin

from sqlalchemy.dialects.postgresql import JSONB

ENTITY_STATUSES = [
    ("standby", "Stand By"),
    ("running", "Running"),
    ("complete", "Complete"),
    ("canceled", "Canceled"),
]


class AssetInstanceLink(db.Model):
    __tablename__ = "asset_instance_link"
    entity_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("entity.id"), primary_key=True
    )
    asset_instance_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("asset_instance.id"),
        primary_key=True,
    )


class EntityLink(db.Model, BaseMixin, SerializerMixin):
    __tablename__ = "entity_link"
    entity_in_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("entity.id"),
        primary_key=True,
        index=True,
    )
    entity_out_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("entity.id"),
        primary_key=True,
        index=True,
    )
    data = db.Column(JSONB)
    nb_occurences = db.Column(db.Integer, default=1)
    label = db.Column(db.String(80), default="")

    __table_args__ = (
        db.UniqueConstraint(
            "entity_in_id",
            "entity_out_id",
            name="entity_link_uc",
        ),
    )

    @classmethod
    def create_from_import(cls, data):
        del data["type"]
        if "project_name" in data:
            del data["project_name"]
        entity_link = cls.get_by(
            entity_in_id=data["entity_in_id"],
            entity_out_id=data["entity_out_id"],
        )
        if entity_link is None:
            return cls.create(**data), False
        else:
            entity_link.update(data)
            return entity_link, True


class EntityConceptLink(db.Model, BaseMixin, SerializerMixin):
    __tablename__ = "entity_concept_link"
    entity_in_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("entity.id"),
        primary_key=True,
    )
    entity_out_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("entity.id"),
        primary_key=True,
    )


class Entity(db.Model, BaseMixin, SerializerMixin):
    """
    Base model to represent assets, shots, sequences, episodes and scenes.
    They have different meaning but they share the same behaviour toward
    tasks and files.
    """

    id = BaseMixin.id
    name = db.Column(db.String(160), nullable=False)
    code = db.Column(db.String(160))  # To store sanitized version of name
    description = db.Column(db.Text())
    shotgun_id = db.Column(db.Integer)
    canceled = db.Column(db.Boolean, default=False)

    nb_frames = db.Column(db.Integer)  # Specific to shots
    nb_entities_out = db.Column(db.Integer, default=0)
    is_casting_standby = db.Column(db.Boolean, default=False)

    is_shared = db.Column(db.Boolean, default=False, nullable=False)

    status = db.Column(
        ChoiceType(ENTITY_STATUSES), default="running", nullable=False
    )

    project_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("project.id"),
        nullable=False,
        index=True,
    )
    entity_type_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("entity_type.id"),
        nullable=False,
        index=True,
    )

    parent_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("entity.id"), index=True
    )  # sequence or episode

    source_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("entity.id"),
        index=True,
        nullable=True,
    )  # if the entity is generated from another one (like shots from scene).

    preview_file_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("preview_file.id", name="fk_main_preview"),
    )
    data = db.Column(JSONB)

    ready_for = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("task_type.id", name="fk_ready_for"),
    )

    created_by = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("person.id"),
        nullable=True,
    )

    entities_out = db.relationship(
        "Entity",
        secondary=EntityLink.__table__,
        primaryjoin=(id == EntityLink.entity_in_id),
        secondaryjoin=(id == EntityLink.entity_out_id),
        backref="entities_in",
    )

    entity_concept_links = db.relationship(
        "Entity",
        secondary=EntityConceptLink.__table__,
        primaryjoin=(id == EntityConceptLink.entity_in_id),
        secondaryjoin=(id == EntityConceptLink.entity_out_id),
        lazy="joined",
    )

    instance_casting = db.relationship(
        "AssetInstance", secondary=AssetInstanceLink.__table__, backref="shots"
    )

    __table_args__ = (
        db.UniqueConstraint(
            "name",
            "project_id",
            "entity_type_id",
            "parent_id",
            name="entity_uc",
        ),
    )

    def set_entities_out(self, entity_ids):
        self.entities_out = []
        for entity_id in entity_ids:
            entity = Entity.get(entity_id)
            if entity is not None:
                self.entities_out.append(entity)
        self.save()

    @classmethod
    def create_from_import(cls, data):
        is_update = False
        previous_entity = cls.get(data["id"])
        (data, entity_ids) = cls.sanitize_import_data(data)

        if previous_entity is None:
            previous_entity = cls.create(**data)
            previous_entity.save()
        else:
            is_update = True
            previous_entity.update(data)
            previous_entity.save()

        if entity_ids is not None:
            previous_entity.set_entities_out(entity_ids)

        return (previous_entity, is_update)

    @classmethod
    def sanitize_import_data(self, data):
        from zou.app.models.preview_file import PreviewFile
        from zou.app.models.entity_type import EntityType

        entity_ids = []
        model_type = data.get("type", "Shot")

        if "entities_out" in data:
            entity_ids = data["entities_out"]
            del data["entities_out"]

        if "asset_type_id" in data:
            data["entity_type_id"] = data["asset_type_id"]
            del data["asset_type_id"]
            del data["asset_type_name"]

        if "sequence_id" in data:
            data["parent_id"] = data["sequence_id"]
            del data["sequence_id"]

        if (
            "preview_file_id" in data
            and data["preview_file_id"] is not None
            and len(data["preview_file_id"]) > 0
        ):
            preview_file = PreviewFile.get(data["preview_file_id"])
            if preview_file is None:
                del data["preview_file_id"]
        elif "preview_file_id" in data:
            del data["preview_file_id"]

        if "frame_in" in data:
            data["data"]["frame_in"] = data["frame_in"]
            del data["frame_in"]

        if "frame_out" in data:
            data["data"]["frame_out"] = data["frame_out"]
            del data["frame_out"]

        if "fps" in data:
            data["data"]["fps"] = data["fps"]
            del data["fps"]

        for field in [
            "entities_in",
            "episode_id",
            "episode_name",
            "project_name",
            "sequence_name",
            "tasks",
            "type",
        ]:
            if field in data:
                del data[field]

        if model_type in ["Shot", "Sequence", "Episode", "Edit"]:
            entity_type = EntityType.get_by(name=model_type)
            data["entity_type_id"] = entity_type.id

        return (data, entity_ids)

    def serialize(self, obj_type=None, relations=False, milliseconds=False):
        serialized_instance = super(Entity, self).serialize(
            obj_type=obj_type,
            relations=relations,
            milliseconds=milliseconds,
        )
        if obj_type == "Concept" and not relations:
            serialized_instance["entity_concept_links"] = [
                str(entity_concept_link.id)
                for entity_concept_link in self.entity_concept_links
            ]
        return serialized_instance


class EntityVersion(db.Model, BaseMixin, SerializerMixin):
    name = db.Column(db.String(160), nullable=False)
    data = db.Column(JSONB)
    entity_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("entity.id"), index=True
    )
    person_id = db.Column(
        UUIDType(binary=False), db.ForeignKey("person.id"), index=True
    )



import traceback
from io import StringIO
from html.parser import HTMLParser
from flask_mail import Message

from zou.app import mail, app


def send_email(subject, html, recipient_email, body=None):
    """
    Send an email with given subject and body to given recipient.
    """
    if body is None:
        body = strip_html_tags(html)
    if app.config["MAIL_DEBUG_BODY"]:
        print(body)
    if app.config["MAIL_ENABLED"]:
        with app.app_context():
            try:
                mail_default_sender = app.config["MAIL_DEFAULT_SENDER"]
                message = Message(
                    sender="Kitsu Bot <%s>" % mail_default_sender,
                    body=body,
                    html=html,
                    subject=subject,
                    recipients=[recipient_email],
                )
                mail.send(message)
            except Exception:
                app.logger.info("Exception when sending a mail notification:")
                app.logger.info(traceback.format_exc())


class HTMLStripper(HTMLParser):
    def __init__(self):
        super().__init__()
        self.reset()
        self.strict = False
        self.convert_charrefs = True
        self.text = StringIO()

    def handle_data(self, d):
        self.text.write(d)

    def get_data(self):
        return self.text.getvalue()


def strip_html_tags(html):
    s = HTMLStripper()
    s.feed(html)
    return s.get_data()



import os
import flask_fs

from zou.app import config

from flask_fs.backends.local import LocalBackend

pictures = None
movies = None
files = None


def path(self, filename):
    folder_one = filename.split("-")[0]
    file_name = "-".join(filename.split("-")[1:])
    folder_two = file_name[:3]
    folder_three = file_name[3:6]

    return os.path.join(
        self.root, folder_one, folder_two, folder_three, file_name
    )


LocalBackend.path = path


def configure_storages(app):
    global pictures, movies, files
    pictures = make_storage("pictures")
    movies = make_storage("movies")
    files = make_storage("files")

    flask_fs.init_app(app, *[pictures, movies, files])


def clear_bucket(bucket):
    for filename in bucket.list_files():
        if isinstance(bucket.backend, LocalBackend):
            folder_one, _, _, file_name = filename.split("/")
            bucket.delete(f"{folder_one}-{file_name}")
        else:
            bucket.delete(filename)


def make_key(prefix, id):
    return f"{prefix}-{id}"


def make_read_generator(bucket, key):
    read_stream = bucket.read_chunks(key)

    def read_generator(read_stream):
        for chunk in read_stream:
            yield chunk

    return read_generator(read_stream)


def make_storage(bucket):
    return flask_fs.Storage(
        "%s%s" % (config.FS_BUCKET_PREFIX, bucket),
        overwrite=True,
    )


def clear():
    clear_bucket(pictures)
    clear_bucket(movies)
    clear_bucket(files)


def add_picture(prefix, id, path):
    key = make_key(prefix, id)
    with open(path, "rb") as fd:
        return pictures.write(key, fd)


def get_picture(prefix, id):
    key = make_key(prefix, id)
    return pictures.read(key)


def open_picture(prefix, id):
    key = make_key(prefix, id)
    return make_read_generator(pictures, key)


def read_picture(prefix, id):
    key = make_key(prefix, id)
    return pictures.read(key)


def exists_picture(prefix, id):
    key = make_key(prefix, id)
    return pictures.exists(key)


def remove_picture(prefix, id):
    key = make_key(prefix, id)
    return pictures.delete(key)


def get_local_picture_path(prefix, id):
    return path(pictures, make_key(prefix, id))


def copy_picture(prefix, id, new_prefix, new_id):
    key = make_key(prefix, id)
    target = make_key(new_prefix, new_id)
    return pictures.copy(key, target)


def add_movie(prefix, id, path):
    key = make_key(prefix, id)
    with open(path, "rb") as fd:
        return movies.write(key, fd)


def get_movie(prefix, id):
    key = make_key(prefix, id)
    return movies.read(key)


def open_movie(prefix, id):
    key = make_key(prefix, id)
    return make_read_generator(movies, key)


def read_movie(prefix, id):
    key = make_key(prefix, id)
    return movies.read(key)


def exists_movie(prefix, id):
    key = make_key(prefix, id)
    return movies.exists(key)


def remove_movie(prefix, id):
    key = make_key(prefix, id)
    return movies.delete(key)


def get_local_movie_path(prefix, id):
    return path(movies, make_key(prefix, id))


def copy_movie(prefix, id, new_prefix, new_id):
    key = make_key(prefix, id)
    target = make_key(new_prefix, new_id)
    return movies.copy(key, target)


def add_file(prefix, id, path):
    key = make_key(prefix, id)
    with open(path, "rb") as fd:
        return files.write(key, fd)


def get_file(prefix, id):
    key = make_key(prefix, id)
    return files.read(key)


def open_file(prefix, id):
    key = make_key(prefix, id)
    return make_read_generator(files, key)


def read_file(prefix, id):
    key = make_key(prefix, id)
    return files.read(key)


def exists_file(prefix, id):
    key = make_key(prefix, id)
    return files.exists(key)


def remove_file(prefix, id):
    key = make_key(prefix, id)
    return files.delete(key)


def get_local_file_path(prefix, id):
    return path(files, make_key(prefix, id))


def copy_file(prefix, id, new_prefix, new_id):
    key = make_key(prefix, id)
    target = make_key(new_prefix, new_id)
    return files.copy(key, target)



import tomlkit
import semver
import email.utils
import spdx_license_list
import zipfile
import importlib
import importlib.util
import sys
import os
import traceback
import shutil

from pathlib import Path
from flask import current_app
from alembic import command
from alembic.config import Config


from pathlib import Path
from collections.abc import MutableMapping


class PluginManifest(MutableMapping):
    def __init__(self, data):
        super().__setattr__("data", data)
        self.validate()

    @classmethod
    def from_plugin_path(cls, path):
        path = Path(path)
        if path.is_dir():
            return cls.from_file(path / "manifest.toml")
        elif zipfile.is_zipfile(path):
            with zipfile.ZipFile(path) as z:
                with z.open("manifest.toml") as f:
                    data = tomlkit.load(f)
            return cls(data)
        else:
            raise ValueError(f"Invalid plugin path: {path}")

    @classmethod
    def from_file(cls, path):
        with open(path, "rb") as f:
            data = tomlkit.load(f)
        return cls(data)

    def write_to_path(self, path):
        path = Path(path)
        with open(path / "manifest.toml", "w", encoding="utf-8") as f:
            tomlkit.dump(self.data, f)

    def validate(self):
        semver.Version.parse(str(self.data["version"]))
        spdx_license_list.LICENSES[self.data["license"]]
        if "maintainer" in self.data:
            name, email_addr = email.utils.parseaddr(self.data["maintainer"])
            self.data["maintainer_name"] = name
            self.data["maintainer_email"] = email_addr

    def to_model_dict(self):
        return {
            "plugin_id": self.data["id"],
            "name": self.data["name"],
            "description": self.data.get("description"),
            "version": str(self.data["version"]),
            "maintainer_name": self.data.get("maintainer_name"),
            "maintainer_email": self.data.get("maintainer_email"),
            "website": self.data.get("website"),
            "license": self.data["license"],
        }

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        self.data[key] = value

    def __delitem__(self, key):
        del self.data[key]

    def __iter__(self):
        return iter(self.data)

    def __len__(self):
        return len(self.data)

    def __repr__(self):
        return f"<PluginManifest {self.data!r}>"

    def __getattr__(self, attr):
        try:
            return self.data[attr]
        except KeyError:
            raise AttributeError(f"'PluginManifest' has no attribute '{attr}'")

    def __setattr__(self, attr, value):
        if attr == "data":
            super().__setattr__(attr, value)
        else:
            self.data[attr] = value


def load_plugin(app, plugin_path, init_plugin=True):
    """
    Load a plugin from the plugin folder.
    """
    plugin_path = Path(plugin_path)
    manifest = PluginManifest.from_plugin_path(plugin_path)

    plugin_module = importlib.import_module(manifest["id"])
    if init_plugin and hasattr(plugin_module, "init_plugin"):
        plugin_module.init_plugin(app, manifest)

    return plugin_module


def load_plugins(app):
    """
    Load plugins from the plugin folder.
    """
    plugin_folder = Path(app.config["PLUGIN_FOLDER"])
    if plugin_folder.exists():
        abs_plugin_path = str(plugin_folder.absolute())
        if abs_plugin_path not in sys.path:
            sys.path.insert(0, abs_plugin_path)

        for plugin_id in os.listdir(plugin_folder):
            try:
                load_plugin(app, plugin_folder / plugin_id)
                app.logger.info(f"Plugin {plugin_id} loaded.")
            except ImportError as e:
                app.logger.error(f"Plugin {plugin_id} failed to import: {e}")
            except Exception as e:
                app.logger.error(
                    f"Plugin {plugin_id} failed to initialize: {e}"
                )
                app.logger.debug(traceback.format_exc())

        if abs_plugin_path in sys.path:
            sys.path.remove(abs_plugin_path)


def migrate_plugin_db(plugin_path, message):
    """
    Generates Alembic migration files in path/migrations.
    """
    plugin_path = Path(plugin_path).absolute()
    models_path = plugin_path / "models.py"

    if not models_path.exists():
        raise FileNotFoundError(f"'models.py' not found in '{plugin_path}'")

    manifest = PluginManifest.from_plugin_path(plugin_path)

    module_name = f"_plugin_models_{manifest['id']}"
    spec = importlib.util.spec_from_file_location(module_name, models_path)
    if spec is None or spec.loader is None:
        raise ImportError(f"Could not load 'models.py' from '{plugin_path}'")

    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    try:
        spec.loader.exec_module(module)
        migrations_dir = plugin_path / "migrations"
        versions_dir = migrations_dir / "versions"
        versions_dir.mkdir(parents=True, exist_ok=True)

        alembic_cfg = Config()
        alembic_cfg.config_file_name = str(
            plugin_path / "migrations" / "alembic.ini"
        )
        alembic_cfg.set_main_option("script_location", str(migrations_dir))
        alembic_cfg.set_main_option(
            "sqlalchemy.url", current_app.config["SQLALCHEMY_DATABASE_URI"]
        )

        command.revision(alembic_cfg, autogenerate=True, message=message)
    finally:
        del sys.modules[module_name]


def run_plugin_migrations(plugin_path, plugin):
    """
    Run plugin migrations.
    """
    plugin_path = Path(plugin_path)

    alembic_cfg = Config()
    alembic_cfg.config_file_name = str(
        plugin_path / "migrations" / "alembic.ini"
    )
    alembic_cfg.set_main_option(
        "script_location", str(plugin_path / "migrations")
    )
    alembic_cfg.set_main_option(
        "sqlalchemy.url", current_app.config["SQLALCHEMY_DATABASE_URI"]
    )

    command.upgrade(alembic_cfg, "head")

    script = command.ScriptDirectory.from_config(alembic_cfg)
    head_revision = script.get_current_head()

    plugin.revision = head_revision

    return head_revision


def downgrade_plugin_migrations(plugin_path):
    """
    Downgrade plugin migrations to base.
    """
    plugin_path = Path(plugin_path)
    manifest = PluginManifest.from_plugin_path(plugin_path)

    alembic_cfg = Config()
    alembic_cfg.config_file_name = str(
        plugin_path / "migrations" / "alembic.ini"
    )
    alembic_cfg.set_main_option(
        "script_location", str(plugin_path / "migrations")
    )
    alembic_cfg.set_main_option(
        "sqlalchemy.url", current_app.config["SQLALCHEMY_DATABASE_URI"]
    )

    try:
        command.downgrade(alembic_cfg, "base")
    except Exception as e:
        current_app.logger.warning(
            f"Downgrade failed for plugin {manifest.id}: {e}"
        )


def create_plugin_package(path, output_path, force=False):
    """
    Create a plugin package.
    """
    path = Path(path)
    if not path.exists():
        raise FileNotFoundError(f"Plugin path '{path}' does not exist.")

    manifest = PluginManifest.from_plugin_path(path)

    output_path = Path(output_path)
    if not output_path.suffix == ".zip":
        output_path /= f"{manifest.id}-{manifest.version}.zip"
    if output_path.exists():
        if force:
            output_path.unlink()
        else:
            raise FileExistsError(
                f"Output path '{output_path}' already exists."
            )

    output_path = shutil.make_archive(
        output_path.with_suffix(""),
        "zip",
        path,
    )
    return output_path


def create_plugin_skeleton(
    path,
    id,
    name,
    description=None,
    version=None,
    maintainer=None,
    website=None,
    license=None,
    force=False,
):
    plugin_template_path = (
        Path(__file__).parent.parent.parent / "plugin_template"
    )
    plugin_path = Path(path) / id

    if plugin_path.exists():
        if force:
            shutil.rmtree(plugin_path)
        else:
            raise FileExistsError(
                f"Plugin '{id}' already exists in {plugin_path}."
            )

    shutil.copytree(plugin_template_path, plugin_path)

    manifest = PluginManifest.from_file(plugin_path / "manifest.toml")

    manifest.id = id
    manifest.name = name
    if description:
        manifest.description = description
    if version:
        manifest.version = version
    if maintainer:
        manifest.maintainer = maintainer
    if website:
        manifest.website = website
    if license:
        manifest.license = license

    manifest.validate()
    manifest.write_to_path(plugin_path)

    return plugin_path


def install_plugin_files(files_path, installation_path):
    """
    Install plugin files.
    """
    files_path = Path(files_path)
    installation_path = Path(installation_path)

    installation_path.mkdir(parents=True, exist_ok=True)

    if files_path.is_dir():
        shutil.copytree(files_path, installation_path, dirs_exist_ok=True)
    elif zipfile.is_zipfile(files_path):
        shutil.unpack_archive(files_path, installation_path, format="zip")
    else:
        raise ValueError(
            f"Plugin path '{files_path}' is not a valid zip file or a directory."
        )

    return installation_path


def uninstall_plugin_files(plugin_path):
    """
    Uninstall plugin files.
    """
    plugin_path = Path(plugin_path)
    if plugin_path.exists():
        shutil.rmtree(plugin_path)
        return True
    return False



from flask import request
from flask_restful import Resource, reqparse
from flask_jwt_extended import jwt_required

from zou.app.utils import permissions

from zou.app.services import (
    chats_service,
    entities_service,
    persons_service,
    user_service,
)
from zou.app.services.exception import WrongParameterException


class ChatResource(Resource):

    @jwt_required()
    def get(self, entity_id):
        """
        Get chat details.
        ---
        tags:
          - Chat
        parameters:
          - in: path
            name: entity_id
            description: ID of the entity related to the chat
            type: integer
            required: true
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
          200:
            description: Chat information
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        user_service.check_entity_access(entity["id"])
        chat = chats_service.get_chat(entity_id)
        chat["messages"] = chats_service.get_chat_messages(entity_id)
        return chat


class ChatMessagesResource(Resource):

    @jwt_required()
    def get(self, entity_id):
        """
        Get chat messages for an entity.
        ---
        tags:
          - Chat
        parameters:
          - in: path
            name: entity_id
            description: ID of the entity related to the chat
            type: integer
            required: true
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Chat messages
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        user_service.check_entity_access(entity["id"])
        return chats_service.get_chat_messages_for_entity(entity_id)

    @jwt_required()
    def post(self, entity_id):
        """
        Create a new chat message.
        ---
        tags:
          - Chat
        parameters:
          - in: path
            name: entity_id
            description: ID of the entity related to the chat
            type: integer
            required: true
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: body
            name: message
            description: Message to send
            type: string
            required: true
            example: Hello, world!
          - in: formData
            name: files
            description: Files to attach
            type: file
            required: false
        responses:
            201:
                description: Chat message created
            400:
                description: Not participant of the chat
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        user_service.check_entity_access(entity["id"])

        person = persons_service.get_current_user()
        if request.is_json:
            location = ["values", "json"]
        else:
            location = ["values", "form"]
        parser = reqparse.RequestParser()
        parser.add_argument(
            "message", type=str, required=True, location=location
        )
        args = parser.parse_args()
        message = args["message"]
        files = request.files

        chat = chats_service.get_chat(entity_id)
        if person["id"] not in chat["participants"]:
            raise WrongParameterException(
                "You are not a participant of this chat"
            )

        return (
            chats_service.create_chat_message(
                chat["id"], person["id"], message, files=files
            ),
            201,
        )


class ChatMessageResource(Resource):

    @jwt_required()
    def get(self, entity_id, chat_message_id):
        """
        Get chat message.
        ---
        tags:
          - Chat
        parameters:
          - in: path
            name: entity_id
            description: ID of the entity related to the chat
            type: integer
            required: true
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: chat_message_id
            description: ID of the chat message
            type: integer
            required: true
            example: 1
        responses:
            200:
                description: Chat message
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        user_service.check_entity_access(entity["id"])
        return chats_service.get_chat_message(chat_message_id)

    @jwt_required()
    def delete(self, entity_id, chat_message_id):
        """
        Delete chat message.
        ---
        tags:
          - Chat
        parameters:
          - in: path
            name: entity_id
            description: ID of the entity related to the chat
            type: integer
            required: true
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: chat_message_id
            description: ID of the chat message
            type: integer
            required: true
            example: 1
        responses:
            204:
                description: Empty response
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        user_service.check_entity_access(entity["id"])

        chat_message = chats_service.get_chat_message(chat_message_id)
        current_user = persons_service.get_current_user()
        if (
            chat_message["person_id"] != current_user["id"]
            or not permissions.has_admin_permissions
        ):
            raise permissions.PermissionDenied
        chats_service.delete_chat_message(chat_message_id)

        return "", 204



"""Add PreviewFile.width and PreviewFile.height

Revision ID: ae0127f2fc56
Revises: f5bdca075cdc
Create Date: 2023-09-13 17:22:11.318041

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "ae0127f2fc56"
down_revision = "f5bdca075cdc"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("preview_file", schema=None) as batch_op:
        batch_op.add_column(sa.Column("width", sa.Integer(), nullable=True))
        batch_op.add_column(sa.Column("height", sa.Integer(), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("preview_file", schema=None) as batch_op:
        batch_op.drop_column("height")
        batch_op.drop_column("width")

    # ### end Alembic commands ###



from zou.app.models.search_filter import SearchFilter

from zou.app.blueprints.crud.base import BaseModelResource, BaseModelsResource


class SearchFiltersResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, SearchFilter)


class SearchFilterResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, SearchFilter)



import datetime


from flask import abort, request
from flask_restful import Resource, inputs
from flask_jwt_extended import jwt_required

from zou.app.services.exception import (
    TaskNotFoundException,
    PersonNotFoundException,
    MalformedFileTreeException,
    WrongDateFormatException,
    WrongParameterException,
)
from zou.app.services import (
    assets_service,
    deletion_service,
    edits_service,
    entities_service,
    files_service,
    file_tree_service,
    notifications_service,
    persons_service,
    preview_files_service,
    projects_service,
    shots_service,
    tasks_service,
    user_service,
    concepts_service,
)
from zou.app.utils import events, query, permissions, date_helpers
from zou.app.mixin import ArgsMixin


class AddPreviewResource(Resource, ArgsMixin):
    @jwt_required()
    def post(self, task_id, comment_id):
        """
        Add preview metadata to given task. The preview file itself should be
        uploaded afterward.

        Revision is automatically set: it is equal to last revision + 1. It can
        be also set manually.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: comment_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: body
            schema:
                type: object
                properties:
                    revision:
                        type: integer
        responses:
            201:
                description: Preview metadata added to given task
        """
        args = self.get_args([("revision", 0, False, int)])

        user_service.check_task_action_access(task_id)

        person = persons_service.get_current_user()
        preview_file = tasks_service.add_preview_file_to_comment(
            comment_id, person["id"], task_id, args["revision"]
        )
        return preview_file, 201


class AddExtraPreviewResource(Resource, ArgsMixin):
    """
    Add a preview to given comment.
    """

    @jwt_required()
    def post(self, task_id, comment_id, preview_file_id):
        """
        Add a preview to given comment.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: comment_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: preview_file_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: formData
            name: file
            type: file
            required: True
        responses:
            201:
                description: Preview added to given comment
        """
        user_service.check_task_action_access(task_id)
        tasks_service.get_comment(comment_id)

        person = persons_service.get_current_user()
        related_preview_file = files_service.get_preview_file(preview_file_id)

        preview_file = tasks_service.add_preview_file_to_comment(
            comment_id, person["id"], task_id, related_preview_file["revision"]
        )
        return preview_file, 201

    @jwt_required()
    def delete(self, task_id, comment_id, preview_file_id):
        """
        Delete preview from given comment.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: comment_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: preview_file_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            204:
                description: Preview deleted from given comment
        """
        task = tasks_service.get_task(task_id)
        user_service.check_project_access(task["project_id"])
        deletion_service.remove_preview_file_by_id(
            preview_file_id, force=self.get_force()
        )
        return "", 204


class TaskPreviewsResource(Resource):
    """
    Return previews linked to given task.
    """

    @jwt_required()
    def get(self, task_id):
        """
        Return previews linked to given task.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Previews linked to given task
        """
        user_service.check_task_access(task_id)
        return files_service.get_preview_files_for_task(task_id)


class TaskCommentsResource(Resource):
    """
    Return comments linked to given task.
    """

    @jwt_required()
    def get(self, task_id):
        """
        Return comments linked to given task.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Comments linked to given task
        """
        user_service.check_task_access(task_id)
        is_client = permissions.has_client_permissions()
        is_manager = permissions.has_manager_permissions()
        is_supervisor = permissions.has_supervisor_permissions()
        return tasks_service.get_comments(
            task_id, is_client, is_manager or is_supervisor
        )


class TaskCommentResource(Resource):
    """
    Remove given comment and update linked task accordingly.
    """

    @jwt_required()
    def get(self, task_id, comment_id):
        """
        Get comment corresponding at given ID.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: comment_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Comment corresponding at given ID
        """
        comment = tasks_service.get_comment(comment_id)
        user_service.check_comment_access(comment)
        return comment

    def pre_delete(self, comment):
        task = tasks_service.get_task(comment["object_id"])
        self.previous_task_status_id = task["task_status_id"]
        return comment

    def post_delete(self, comment):
        task = tasks_service.get_task(comment["object_id"])
        self.new_task_status_id = task["task_status_id"]
        if self.previous_task_status_id != self.new_task_status_id:
            events.emit(
                "task:status-changed",
                {
                    "task_id": task["id"],
                    "new_task_status_id": self.new_task_status_id,
                    "previous_task_status_id": self.previous_task_status_id,
                    "person_id": comment["person_id"],
                },
                project_id=task["project_id"],
            )
        return comment

    @jwt_required()
    def delete(self, task_id, comment_id):
        """
        Delete a comment corresponding at given ID.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: comment_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            204:
                description: Comment corresponding at given ID deleted
        """
        comment = tasks_service.get_comment(comment_id)
        task = tasks_service.get_task(comment["object_id"])
        if permissions.has_manager_permissions():
            user_service.check_project_access(task["project_id"])
        else:
            user_service.check_person_access(comment["person_id"])
        self.pre_delete(comment)
        deletion_service.remove_comment(comment_id)
        tasks_service.reset_task_data(comment["object_id"])
        tasks_service.clear_comment_cache(comment_id)
        self.post_delete(comment)
        return "", 204


class PersonTasksResource(Resource):
    """
    Return task assigned to given user of which status has is_done flag sets
    to false.
    """

    @jwt_required()
    def get(self, person_id):
        """
        Return task assigned to given user of which status has is_done flag sets to false.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: person_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Tasks assigned to user that are not done
        """
        user_service.check_person_is_not_bot(person_id)
        if not permissions.has_admin_permissions():
            projects = user_service.related_projects()
        else:
            projects = projects_service.open_projects()
        if permissions.has_vendor_permissions():
            person = persons_service.get(person_id)
            if person["role"] == "vendor":
                return []
        elif permissions.has_client_permissions():
            return []
        return tasks_service.get_person_tasks(person_id, projects)


class PersonRelatedTasksResource(Resource):
    """
    For all entities assigned to given person (that have at least one task
    assigned to given person), returns all tasks for given task type.
    """

    @jwt_required()
    def get(self, person_id, task_type_id):
        """
        For all entities assigned to given person (that have at least one task assigned to given person), returns all tasks for given task type.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: person_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All Tasks for given task type
        """
        user_service.check_person_is_not_bot(person_id)
        user = persons_service.get_current_user()
        if person_id != user["id"]:
            permissions.check_admin_permissions()
        return tasks_service.get_person_related_tasks(person_id, task_type_id)


class PersonDoneTasksResource(Resource):
    """
    Return task assigned to given user of which status has is_done flag sets
    to true. It return only tasks related to open projects.
    """

    @jwt_required()
    def get(self, person_id):
        """
        Return task assigned to given user of which status has is_done flag sets to true.
        ---
        tags:
        - Tasks
        description: It return only tasks related to open projects.
        parameters:
          - in: path
            name: person_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Tasks assigned to user that are done
        """
        user_service.check_person_is_not_bot(person_id)
        if not permissions.has_admin_permissions():
            projects = user_service.related_projects()
        else:
            projects = projects_service.open_projects()
        if permissions.has_vendor_permissions():
            person = persons_service.get(person_id)
            if person["role"] == "vendor":
                return []
        elif permissions.has_client_permissions():
            return []
        return tasks_service.get_person_done_tasks(person_id, projects)


class CreateShotTasksResource(Resource):
    """
    Create a new task for given shot and task type.
    """

    @jwt_required()
    def post(self, project_id, task_type_id):
        """
        Create a new task for given shot and task type.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            201:
                description: New task for given shot and task type created
        """
        user_service.check_manager_project_access(project_id)
        task_type = tasks_service.get_task_type(task_type_id)

        shot_ids = request.json
        shots = []
        if isinstance(shot_ids, list) and len(shot_ids) > 0:
            for shot_id in shot_ids:
                shot = shots_service.get_shot(shot_id)
                if shot["project_id"] == project_id:
                    shots.append(shot)
        else:
            criterions = query.get_query_criterions_from_request(request)
            criterions["project_id"] = project_id
            shots = shots_service.get_shots(criterions)

        task_type = tasks_service.get_task_type(task_type_id)
        tasks = tasks_service.create_tasks(task_type, shots)
        return tasks, 201


class CreateConceptTasksResource(Resource):
    """
    Create a new task for given concept and task type.
    """

    @jwt_required()
    def post(self, project_id, task_type_id):
        """
        Create a new task for given concept and task type.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            201:
                description: New task for given concept and task type created
        """
        user_service.check_project_access(project_id)
        if (
            permissions.has_vendor_permissions()
            or permissions.has_client_permissions()
        ):
            raise permissions.PermissionDenied
        task_type = tasks_service.get_task_type(task_type_id)

        concept_ids = request.json
        concepts = []
        if isinstance(concept_ids, list) and len(concept_ids) > 0:
            for concept_id in concept_ids:
                concept = concepts_service.get_concept(concept_id)
                if concept["project_id"] == project_id:
                    concepts.append(concept)
        else:
            criterions = query.get_query_criterions_from_request(request)
            criterions["project_id"] = project_id
            concepts = concepts_service.get_concepts(criterions)

        for concept in concepts:
            user_service.check_entity_access(concept["id"])

        task_type = tasks_service.get_task_type(task_type_id)
        tasks = tasks_service.create_tasks(task_type, concepts)
        return tasks, 201


class CreateEntityTasksResource(Resource):
    @jwt_required()
    def post(self, project_id, entity_type, task_type_id):
        """
        Create a new task with given task type for each entity of given
        entity type.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            201:
                description: List of created tasks.
        """
        user_service.check_manager_project_access(project_id)
        task_type = tasks_service.get_task_type(task_type_id)
        entity_type_dict = (
            entities_service.get_entity_type_by_name_or_not_found(
                entity_type.capitalize()
            )
        )

        entity_ids = request.json
        entities = []
        if isinstance(entity_ids, list) and len(entity_ids) > 0:
            for entity_id in entity_ids:
                entity = entities_service.get_entity(entity_id)
                if entity["project_id"] == project_id:
                    entities.append(entity)
        else:
            criterions = query.get_query_criterions_from_request(request)
            episode_id = criterions.get("episode_id", None)
            entities = entities_service.get_entities_for_project(
                project_id, entity_type_dict["id"], episode_id=episode_id
            )

        tasks = tasks_service.create_tasks(task_type, entities)
        return tasks, 201


class CreateAssetTasksResource(Resource):
    """
    Create a new task for given asset and task type.
    """

    @jwt_required()
    def post(self, project_id, task_type_id):
        """
        Create a new task for given asset and task type.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            201:
                description: New task for given asset and task type created
        """
        user_service.check_manager_project_access(project_id)
        task_type = tasks_service.get_task_type(task_type_id)

        asset_ids = request.json
        assets = []
        if isinstance(asset_ids, list) and len(asset_ids) > 0:
            for asset_id in asset_ids:
                asset = assets_service.get_asset(asset_id)
                if asset["project_id"] == project_id:
                    assets.append(asset)
        else:
            criterions = query.get_query_criterions_from_request(request)
            criterions["project_id"] = project_id
            assets = assets_service.get_assets(criterions)

        tasks = tasks_service.create_tasks(task_type, assets)
        return tasks, 201


class CreateEditTasksResource(Resource):
    """
    Create a new task for given edit and task type.
    """

    @jwt_required()
    def post(self, project_id, task_type_id):
        """
        Create a new task for given edit and task type.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            201:
                description: New task for given edit and task type created
        """
        user_service.check_manager_project_access(project_id)
        task_type = tasks_service.get_task_type(task_type_id)

        edit_ids = request.json
        edits = []
        if isinstance(edit_ids, list) and len(edit_ids) > 0:
            for edit_id in edit_ids:
                edit = edits_service.get_edit(edit_id)
                if edit["project_id"] == project_id:
                    edits.append(edit)
        else:
            criterions = query.get_query_criterions_from_request(request)
            criterions["project_id"] = project_id
            edits = edits_service.get_edits(criterions)

        tasks = tasks_service.create_tasks(task_type, edits)
        return tasks, 201


class ToReviewResource(Resource, ArgsMixin):
    """
    Change a task status to "to review". It creates a new preview file entry
    and set path from the hard disk.
    """

    @jwt_required()
    def put(self, task_id):
        """
        Change a task status to "to review".
        ---
        tags:
        - Tasks
        description: It creates a new preview file entry and set path from the hard disk.
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: body
            name: Task
            description: person ID, name, comment, revision and change status of task
            schema:
                type: object
                properties:
                    person_id:
                        type: string
                        format: uuid
                        example: a24a6ea4-ce75-4665-a070-57453082c25
                    comment:
                        type: string
                    name:
                        type: string
                    revision:
                        type: integer
                    change_status:
                        type: boolean
        responses:
            200:
                description: Task status changed to "to review"
            400:
                description: Given person not found
        """
        (
            person_id,
            comment,
            name,
            revision,
            change_status,
        ) = self.get_arguments()

        try:
            task = tasks_service.get_task(task_id)
            user_service.check_project_access(task["project_id"])
            user_service.check_entity_access(task["entity_id"])

            if person_id is not None:
                person = persons_service.get_person(person_id)
            else:
                person = persons_service.get_current_user()

            preview_path = self.get_preview_path(task, name, revision)

            task = tasks_service.task_to_review(
                task_id, person, comment, preview_path, change_status
            )
        except PersonNotFoundException:
            return {"error": True, "message": "Cannot find given person."}, 400

        return task

    def get_preview_path(self, task, name, revision):
        try:
            folder_path = file_tree_service.get_working_folder_path(
                task, name=name, mode="preview", revision=revision
            )
            file_name = file_tree_service.get_working_file_name(
                task, name=name, mode="preview", revision=revision
            )
        except MalformedFileTreeException:  # No template for preview files.
            return {"folder_path": "", "file_name": ""}

        return {"folder_path": folder_path, "file_name": file_name}

    def get_arguments(self):
        args = self.get_args(
            [
                "person_id",
                ("comment", ""),
                ("name", "main"),
                {"name": "revision", "default": 1, "type": int},
                {
                    "name": "change_status",
                    "default": True,
                    "type": inputs.boolean,
                },
            ]
        )

        return (
            args["person_id"],
            args["comment"],
            args["name"],
            args["revision"],
            args["change_status"],
        )


class ClearAssignationResource(Resource, ArgsMixin):
    """
    Remove all assignations set to given task.
    """

    @jwt_required()
    def put(self):
        """
        Remove all assignations set to given task.
        ---
        tags:
        - Tasks
        parameters:
          - in: body
            name: Task
            description: List of tasks ID and person ID
            schema:
                type: object
                required:
                  - task_ids
                properties:
                    task_ids:
                        type: string
                        format: uuid
                        example: a24a6ea4-ce75-4665-a070-57453082c25
                    person_id:
                        type: string
                        format: uuid
                        example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All assignations removed
        """
        (task_ids, person_id) = self.get_arguments()

        tasks = []
        for task_id in task_ids:
            try:
                user_service.check_task_department_access_for_unassign(
                    task_id, person_id
                )
                tasks_service.clear_assignation(task_id, person_id=person_id)
                tasks.append(task_id)
            except permissions.PermissionDenied:
                pass
            except TaskNotFoundException:
                pass

        return tasks

    def get_arguments(self):
        args = self.get_args(
            [
                {
                    "name": "task_ids",
                    "help": "Tasks list required.",
                    "required": True,
                    "action": "append",
                },
                "person_id",
            ]
        )

        return args["task_ids"], args["person_id"]


class TasksAssignResource(Resource, ArgsMixin):
    """
    Assign given task lists to given person. If a given task ID is wrong,
    it ignores it.
    """

    @jwt_required()
    def put(self, person_id):
        """
        Assign given task lists to given person.
        ---
        tags:
        - Tasks
        description: If a given task ID is wrong, it ignores it.
        parameters:
          - in: path
            name: person_id
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
            required: True
          - in: body
            name: Task
            description: List of tasks ID
            schema:
                type: object
                required:
                  - task_ids
                properties:
                    task_ids:
                        type: string
                        format: uuid
                        example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Given tasks lists assigned to given person
        """
        args = self.get_args(
            [
                {
                    "name": "task_ids",
                    "help": "Tasks list required.",
                    "required": True,
                    "action": "append",
                },
            ]
        )

        tasks = []
        current_user = persons_service.get_current_user()
        for task_id in args["task_ids"]:
            try:
                user_service.check_person_is_not_bot(person_id)
                user_service.check_task_department_access(task_id, person_id)
                task = tasks_service.assign_task(
                    task_id, person_id, current_user["id"]
                )
                notifications_service.create_assignation_notification(
                    task_id, person_id
                )
                tasks.append(task)
            except TaskNotFoundException:
                pass
            except permissions.PermissionDenied:
                pass
            except PersonNotFoundException:
                return {"error": "Assignee doesn't exist in database."}, 400
        if len(tasks) > 0:
            projects_service.add_team_member(tasks[0]["project_id"], person_id)

        return tasks


class TaskAssignResource(Resource, ArgsMixin):
    """
    Assign given task to given person.
    """

    @jwt_required()
    def put(self, task_id):
        """
        Assign given task list to given person.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: body
            name: Person
            description: Person ID
            schema:
                type: object
                required:
                  - person_id
                properties:
                    person_id:
                        type: string
                        format: uuid
                        example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Given task assigned to given person
            400:
                description: Assignee non-existent in database
        """
        args = self.get_args(
            [
                {
                    "name": "person_id",
                    "help": "Assignee ID required.",
                    "required": True,
                },
            ]
        )
        person_id = args["person_id"]
        current_user = persons_service.get_current_user()
        try:
            user_service.check_person_is_not_bot(person_id)
            user_service.check_task_department_access(task_id, person_id)
            task = tasks_service.assign_task(
                task_id, person_id, current_user["id"]
            )
            notifications_service.create_assignation_notification(
                task_id, person_id
            )
            projects_service.add_team_member(task["project_id"], person_id)
        except PersonNotFoundException:
            return {"error": "Assignee doesn't exist in database."}, 400

        return task


class TaskFullResource(Resource):
    """
    Return a task with many information: full details for assignees, full
    details for task type, full details for task status, etc.
    """

    @jwt_required()
    def get(self, task_id):
        """
        Return a task with many information.
        ---
        description: Full details for assignees, full details for task type, full details for task status, etc.
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Task with many information
        """
        task = tasks_service.get_full_task(
            task_id, persons_service.get_current_user()["id"]
        )
        user_service.check_project_access(task["project_id"])
        user_service.check_entity_access(task["entity_id"])
        return task


class TaskForEntityResource(Resource):
    """
    Return tasks related to given entity asset, episode, sequence, shot or
    scene.
    """

    @jwt_required()
    def get(self, entity_id, task_type_id):
        """
        Return tasks related to given entity asset, episode, sequence, shot or scene.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: entity_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Tasks related to given entity asset, episode, sequence, shot or scene
        """
        entity = entities_service.get_entity(entity_id)
        user_service.check_project_access(entity["project_id"])
        return tasks_service.get_tasks_for_entity_and_task_type(
            entity_id, task_type_id
        )


class SetTimeSpentResource(Resource, ArgsMixin):
    """
    Set time spent by a person on a task for a given day.
    """

    @jwt_required()
    def post(self, task_id, date, person_id):
        """
        Set time spent by a person on a task for a given day.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: date
            required: True
            type: string
            format: date
            example: "2022-07-12"
          - in: path
            name: person_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: body
            name: Duration
            schema:
                type: object
                properties:
                    duration:
                        type: float
        responses:
            201:
                description: Time spent by given person on given task for given day is set
            404:
                description: Wrong date format
        """
        user_service.check_person_is_not_bot(person_id)
        args = self.get_args([("duration", 0, True, int)])
        try:
            if args["duration"] <= 0:
                raise WrongParameterException("Duration must be positive")
            user_service.check_time_spent_access(task_id, person_id)
            time_spent = tasks_service.create_or_update_time_spent(
                task_id,
                person_id,
                date_helpers.get_date_from_string(date),
                args["duration"],
            )
            return time_spent, 201
        except ValueError:
            abort(404)
        except WrongDateFormatException:
            abort(404)

    @jwt_required()
    def delete(self, task_id, date, person_id):
        """
        Delete time spent by a person on a task for a given day.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: date
            required: True
            type: string
            format: date
            example: "2022-07-12"
          - in: path
            name: person_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            201:
                description: Time spent by given person on given task for given day is removed
            404:
                description: Wrong date format
        """
        user_service.check_person_is_not_bot(person_id)
        try:
            user_service.check_time_spent_access(task_id, person_id)
            time_spent = tasks_service.delete_time_spent(
                task_id,
                person_id,
                datetime.datetime.strptime(date, "%Y-%m-%d"),
            )
            return time_spent, 201
        except ValueError:
            abort(404)
        except WrongDateFormatException:
            abort(404)


class AddTimeSpentResource(Resource, ArgsMixin):
    """
    Add given timeframe to time spent by a person on a task for a given day.
    """

    @jwt_required()
    def post(self, task_id, date, person_id):
        """
        Add given timeframe to time spent by a person on a task for a given day.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: date
            required: True
            type: string
            format: date
            example: "2022-07-12"
          - in: path
            name: person_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: body
            name: Duration
            schema:
                type: object
                properties:
                    duration:
                        type: float
        responses:
            201:
                description: Given timeframe added to time spent by given person on given task for given day
            404:
                description: Wrong date format
        """
        user_service.check_person_is_not_bot(person_id)
        args = self.get_args([("duration", 0, True, int)])
        try:
            if args["duration"] <= 0:
                raise WrongParameterException("Duration must be positive")
            user_service.check_time_spent_access(task_id, person_id)
            time_spent = tasks_service.create_or_update_time_spent(
                task_id, person_id, date, args["duration"], add=True
            )
            return time_spent, 201
        except ValueError:
            abort(404)
        except WrongDateFormatException:
            abort(404)


class GetTimeSpentResource(Resource):
    """
    Get time spent on a given task.
    """

    @jwt_required()
    def get(self, task_id):
        """
        Get time spent on a given task.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Time spent on given task
            404:
                description: Wrong date format
        """
        user_service.check_task_access(task_id)
        return tasks_service.get_time_spents(task_id)


class GetTimeSpentDateResource(Resource):
    """
    Get time spent on a given task and date.
    """

    @jwt_required()
    def get(self, task_id, date):
        """
        Get time spent on a given task and date.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: task_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: date
            required: True
            type: string
            format: date
            example: "2022-07-12"
        responses:
            200:
                description: Time spent on given task and date
            404:
                description: Wrong date format
        """
        try:
            user_service.check_task_access(task_id)
            return tasks_service.get_time_spents(task_id, date)
        except WrongDateFormatException:
            abort(404)


class DeleteAllTasksForTaskTypeResource(Resource):
    """
    Delete all tasks for a given task type and project. It's mainly used
    when tasks are created by mistake at the beginning of the project.
    """

    @jwt_required()
    def delete(self, project_id, task_type_id):
        """
        Delete all tasks for a given task type and project.
        ---
        tags:
        - Tasks
        description: It's mainly used when tasks are created by mistake at the beginning of the project.
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: path
            name: task_type_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            204:
                description: All tasks for given task type and project deleted
        """
        permissions.check_admin_permissions()
        projects_service.get_project(project_id)
        task_ids = deletion_service.remove_tasks_for_project_and_task_type(
            project_id, task_type_id
        )
        for task_id in task_ids:
            tasks_service.clear_task_cache(task_id)
        return "", 204


class DeleteTasksResource(Resource):
    """
    Delete tasks matching id list given in parameter. See it as a way to batch
    delete tasks.
    """

    @jwt_required()
    def post(self, project_id):
        """
        Delete tasks matching id list given in parameter.
        ---
        tags:
        - Tasks
        description: See it as a way to batch delete tasks.
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Tasks matching id list given in parameter deleted
        """
        user_service.check_manager_project_access(project_id)
        task_ids = request.json
        task_ids = deletion_service.remove_tasks(project_id, task_ids)
        for task_id in task_ids:
            tasks_service.clear_task_cache(task_id)
        return task_ids, 200


class ProjectSubscriptionsResource(Resource):
    """
    Retrieve all subcriptions to tasks related to given project.
    It's mainly used for synchronisation purpose.
    """

    @jwt_required()
    @permissions.require_admin
    def get(self, project_id):
        """
        Retrieve all subcriptions to tasks related to given project.
        ---
        tags:
        - Tasks
        description: It's mainly used for synchronisation purpose.
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All subcriptions to tasks related to given project
        """
        projects_service.get_project(project_id)
        return notifications_service.get_subscriptions_for_project(project_id)


class ProjectNotificationsResource(Resource, ArgsMixin):
    """
    Retrieve all notifications related to given project.
    It's mainly used for synchronisation purpose.
    """

    @jwt_required()
    @permissions.require_admin
    def get(self, project_id):
        """
        Retrieve all notifications to tasks related to given project.
        ---
        tags:
        - Tasks
        description: It's mainly used for synchronisation purpose.
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All notifications to tasks related to given project
        """
        projects_service.get_project(project_id)
        page = self.get_page()
        return notifications_service.get_notifications_for_project(
            project_id, page
        )


class ProjectTasksResource(Resource, ArgsMixin):
    """
    Retrieve all tasks related to given project.
    It's mainly used for synchronisation purpose.
    """

    @jwt_required()
    def get(self, project_id):
        """
        Retrieve all tasks related to given project.
        ---
        tags:
        - Tasks
        description: It's mainly used for synchronisation purpose.
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: query
            name: page
            required: False
            type: integer
            example: 1
          - in: query
            name: task_type_id
            required: False
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: query
            name: episode_id
            required: False
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: All tasks related to given project
        """
        projects_service.get_project(project_id)
        page = self.get_page()
        task_type_id = self.get_task_type_id()
        episode_id = self.get_episode_id()
        return tasks_service.get_tasks_for_project(
            project_id, page, task_type_id=task_type_id, episode_id=episode_id
        )


class ProjectCommentsResource(Resource, ArgsMixin):
    """
    Retrieve all comments to tasks related to given project.
    It's mainly used for synchronisation purpose.
    """

    @jwt_required()
    def get(self, project_id):
        """
        Retrieve all comments to tasks related to given project.
        ---
        tags:
        - Tasks
        description: It's mainly used for synchronisation purpose.
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: query
            name: limit
            type: integer
            default: 100
            example: 100
        responses:
            200:
                description: All comments to tasks related to given project
        """
        projects_service.get_project(project_id)
        user_service.check_project_access(project_id)
        if (
            permissions.has_vendor_permissions()
            or permissions.has_client_permissions()
        ):
            raise permissions.PermissionDenied
        page = self.get_page()
        limit = self.get_limit()
        return tasks_service.get_comments_for_project(project_id, page, limit)


class ProjectPreviewFilesResource(Resource, ArgsMixin):
    """
    Preview files related to a given project.
    """

    @jwt_required()
    @permissions.require_admin
    def get(self, project_id):
        """
        Preview files related to a given project.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Preview files related to given project
        """
        projects_service.get_project(project_id)
        page = self.get_page()
        return files_service.get_preview_files_for_project(project_id, page)


class SetTaskMainPreviewResource(Resource):
    @jwt_required()
    def put(self, task_id):
        """
        Set last preview from given task as main preview of the related entity.
        This preview will be used as thumbnail to illustrate the entity.
        ---
        tags:
          - Tasks
        description: This preview will be used to illustrate the entity.
        parameters:
          - in: path
            name: preview_file_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: Given preview set as main preview
        """
        task = tasks_service.get_task(task_id)
        user_service.check_project_access(task["project_id"])
        user_service.check_entity_access(task["entity_id"])
        preview_file = preview_files_service.get_last_preview_file_for_task(
            task_id
        )
        if preview_file is not None:
            entity = entities_service.update_entity_preview(
                task["entity_id"], preview_file["id"]
            )
        return entity


class PersonsTasksDatesResource(Resource, ArgsMixin):
    @jwt_required()
    @permissions.require_admin
    def get(self):
        """
        For schedule usages, for each active person, it returns the first start
        date of all tasks of assigned to this person and the last end date.
        ---
        tags:
        - Tasks
        parameters:
          - in: path
            name: project_id
            required: True
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
        responses:
            200:
                description: For each person, the first start date of all tasks of assigned to this person and the last end date.
        """
        permissions.check_admin_permissions()
        args = self.get_args([("project_id", None, True, str)])
        return tasks_service.get_persons_tasks_dates(
            project_id=args["project_id"]
        )


class OpenTasksResource(Resource, ArgsMixin):
    """
    Return all tasks related to open projects.
    """

    @jwt_required()
    def get(self):
        """
        Return all tasks related to open projects.
        ---
        tags:
        - Tasks
        parameters:
          - in: query
            name: project_id
            description: Filter tasks on given project ID
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: query
            name: task_status_id
            description: Filter tasks on given task status ID
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: query
            name: task_type_id
            description: Filter tasks on given task type ID ID
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: query
            name: person_id
            description: Filter tasks on given person ID
            type: string
            format: uuid
            example: a24a6ea4-ce75-4665-a070-57453082c25
          - in: query
            name: start_date
            description: Filter tasks posterior to given start date
            type: string
            format: date
            example: "2022-07-12"
          - in: query
            name: due_date
            description: Filter tasks anterior to given due date
            type: string
            format: date
            example: "2022-07-12"
          - in: query
            name: priority
            description: Filter tasks on given priority
            type: integer
            example: "3"
          - in: query
            name: page
            description: Page number
            type: integer
            example: 1
            default: 1
          - in: query
            name: limit
            description: Number of tasks per page
            type: integer
            example: 100
            default: 100

        responses:
            200:
                schema:
                    type: object
                    properties:
                        data:
                            type: array
                            description: List of tasks
                        stats:
                            type: object
                            properties:
                                total:
                                    type: integer
                                    description: Total number of tasks
                                total_duration:
                                    type: integer
                                    description: Total duration of tasks in minutes
                                total_estimation:
                                    type: integer
                                    description: Total estimation of tasks in minutes
                                status:
                                    type: object
                                    description: Number of tasks per status
                        limit:
                            type: integer
                            description: Number of tasks per page
                        page:
                            type: integer
                            description: Page number
                        is_more:
                            type: boolean
                            description: True if there are more tasks to retrieve
        """
        args = self.get_args(
            [
                ("task_type_id", None, False, str),
                ("project_id", None, False, str),
                ("person_id", None, False, str),
                ("task_status_id", None, False, str),
                ("studio_id", None, False, str),
                ("department_id", None, False, str),
                ("start_date", None, False, str),
                ("due_date", None, False, str),
                ("priority", None, False, str),
                ("group_by", None, False, str),
                ("page", None, False, int),
                ("limit", 100, False, int),
            ]
        )
        return tasks_service.get_open_tasks(
            task_type_id=args["task_type_id"],
            project_id=args["project_id"],
            person_id=args["person_id"],
            task_status_id=args["task_status_id"],
            studio_id=args["studio_id"],
            department_id=args["department_id"],
            start_date=args["start_date"],
            due_date=args["due_date"],
            priority=args["priority"],
            page=args["page"],
            limit=args["limit"],
        )


class OpenTasksStatsResource(Resource, ArgsMixin):

    @jwt_required()
    def get(self):
        """
        Return task amount, task done amount, total estimation, total duration
        by status by task type by project for open projects. It aggregates the
        result at the project level.
        ---
        tags:
        - Tasks
        responses:
            200:
                description: A dict organized by project that contains
                            the results for each task type and status pairs.
        """
        return tasks_service.get_open_tasks_stats()



from zou.app import db
from zou.app.models.serializer import SerializerMixin
from zou.app.models.base import BaseMixin

from sqlalchemy_utils import UUIDType


class ProductionScheduleVersion(db.Model, BaseMixin, SerializerMixin):
    """
    Describe a production schedule.
    """

    name = db.Column(db.String(80), nullable=False)
    project_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("project.id", ondelete="CASCADE"),
        index=True,
        nullable=False,
    )
    production_schedule_from = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("production_schedule_version.id"),
        index=True,
        nullable=True,
    )
    locked = db.Column(db.Boolean(), default=False)
    canceled = db.Column(db.Boolean(), default=False)
    __table_args__ = (
        db.UniqueConstraint(
            "name", "project_id", name="production_schedule_version_uc"
        ),
    )


class ProductionScheduleVersionTaskLinkPersonLink(db.Model):
    __tablename__ = "production_schedule_version_task_link_person_link"
    production_schedule_version_task_link_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey(
            "production_schedule_version_task_link.id", ondelete="CASCADE"
        ),
        primary_key=True,
    )
    person_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("person.id", ondelete="CASCADE"),
        primary_key=True,
    )


class ProductionScheduleVersionTaskLink(db.Model, BaseMixin, SerializerMixin):
    """
    Link a task to a production schedule version.
    """

    production_schedule_version_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("production_schedule_version.id", ondelete="CASCADE"),
        index=True,
        nullable=False,
    )
    task_id = db.Column(
        UUIDType(binary=False),
        db.ForeignKey("task.id", ondelete="CASCADE"),
        index=True,
        nullable=False,
    )
    start_date = db.Column(db.DateTime)
    due_date = db.Column(db.DateTime)
    estimation = db.Column(db.Float, default=0)
    assignees = db.relationship(
        "Person",
        secondary=ProductionScheduleVersionTaskLinkPersonLink.__table__,
    )
    __table_args__ = (
        db.UniqueConstraint(
            "production_schedule_version_id",
            "task_id",
            name="production_schedule_version_task_link_uc",
        ),
    )



"""empty message

Revision ID: 4e3738cdc34c
Revises: 6c597e842afa
Create Date: 2021-07-30 15:22:32.641313

"""

from sqlalchemy.orm.session import Session
from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils
import uuid
from sqlalchemy.ext.declarative import declarative_base
from zou.migrations.utils.base import BaseMixin

# revision identifiers, used by Alembic.
revision = "4e3738cdc34c"
down_revision = "6c597e842afa"
branch_labels = None
depends_on = None

base = declarative_base()


class ProjectTaskTypeLink(base, BaseMixin):
    __tablename__ = "project_task_type_link"
    project_id = sa.Column(
        sqlalchemy_utils.UUIDType(binary=False),
        primary_key=True,
    )
    task_type_id = sa.Column(
        sqlalchemy_utils.UUIDType(binary=False),
        primary_key=True,
    )
    priority = sa.Column(sa.Integer, default=None)

    __table_args__ = (
        sa.UniqueConstraint(
            "project_id", "task_type_id", name="project_tasktype_uc"
        ),
    )


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "project_task_type_link",
        sa.Column("created_at", sa.DateTime(), nullable=True),
    )
    op.add_column(
        "project_task_type_link",
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(binary=False),
            default=uuid.uuid4,
            nullable=True,
        ),
    )
    op.execute(f"UPDATE project_task_type_link SET id = '{uuid.uuid4()}'")
    op.add_column(
        "project_task_type_link",
        sa.Column("updated_at", sa.DateTime(), nullable=True),
    )
    op.add_column(
        "project_task_type_link",
        sa.Column("priority", sa.Integer(), nullable=True),
    )
    session = Session(bind=op.get_bind())
    for link in session.query(ProjectTaskTypeLink).all():
        link.id = uuid.uuid4()
        session.add(link)
        session.commit()
    op.alter_column("project_task_type_link", "id", nullable=False)
    op.create_unique_constraint(
        "project_tasktype_uc",
        "project_task_type_link",
        ["project_id", "task_type_id"],
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "project_tasktype_uc", "project_task_type_link", type_="unique"
    )
    op.drop_column("project_task_type_link", "priority")
    op.drop_column("project_task_type_link", "updated_at")
    op.drop_column("project_task_type_link", "id")
    op.drop_column("project_task_type_link", "created_at")
    # ### end Alembic commands ###



from flask import Blueprint

from zou.app.utils.api import configure_api_from_blueprint

from zou.app.blueprints.comments.resources import (
    AckCommentResource,
    AddAttachmentToCommentResource,
    AttachmentResource,
    CommentTaskResource,
    CommentManyTasksResource,
    DownloadAttachmentResource,
    ProjectAttachmentFiles,
    TaskAttachmentFiles,
    ReplyCommentResource,
    DeleteReplyCommentResource,
)


routes = [
    ("/data/tasks/<task_id>/comments/<comment_id>/ack", AckCommentResource),
    (
        "/data/tasks/<task_id>/comments/<comment_id>/reply",
        ReplyCommentResource,
    ),
    (
        "/data/tasks/<task_id>/comments/<comment_id>/attachments/<attachment_id>",
        AttachmentResource,
    ),
    (
        "/data/tasks/<task_id>/comments/<comment_id>/reply/<reply_id>",
        DeleteReplyCommentResource,
    ),
    (
        "/data/attachment-files/<attachment_file_id>/file/<file_name>",
        DownloadAttachmentResource,
    ),
    (
        "/actions/tasks/<task_id>/comments/<comment_id>/add-attachment",
        AddAttachmentToCommentResource,
    ),
    ("/data/projects/<project_id>/attachment-files", ProjectAttachmentFiles),
    ("/data/tasks/<task_id>/attachment-files", TaskAttachmentFiles),
    ("/actions/tasks/<task_id>/comment", CommentTaskResource),
    (
        "/actions/projects/<project_id>/tasks/comment-many",
        CommentManyTasksResource,
    ),
]

blueprint = Blueprint("comments", "comments")
api = configure_api_from_blueprint(blueprint, routes)



"""add exceptions to budget entries

Revision ID: e7e633bd6fa2
Revises: 4aab1f84ad72
Create Date: 2025-05-12 18:47:21.506153

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "e7e633bd6fa2"
down_revision = "4aab1f84ad72"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("budget_entry", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column(
                "exceptions",
                postgresql.JSONB(astext_type=sa.Text()),
                nullable=True,
            )
        )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("budget_entry", schema=None) as batch_op:
        batch_op.drop_column("exceptions")
    # ### end Alembic commands ###



"""Add day_off new columns

Revision ID: 9b85c14fa8a7
Revises: 16328eae4b5f
Create Date: 2024-02-23 14:48:48.461237

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.orm.session import Session
from zou.migrations.utils.base import BaseMixin
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy_utils import UUIDType


# revision identifiers, used by Alembic.
revision = "9b85c14fa8a7"
down_revision = "16328eae4b5f"
branch_labels = None
depends_on = None

base = declarative_base()


class DayOff(base, BaseMixin):
    """
    Tells that someone will have a day off this day.
    """

    __tablename__ = "day_off"
    date = sa.Column(sa.Date, nullable=False)
    end_date = sa.Column(sa.Date, nullable=True)
    description = sa.Column(sa.Text)
    person_id = sa.Column(
        UUIDType(binary=False), sa.ForeignKey("person.id"), index=True
    )
    __table_args__ = (
        sa.UniqueConstraint("person_id", "date", name="day_off_uc"),
        sa.CheckConstraint("date <= end_date", name="day_off_date_check"),
    )


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("day_off", schema=None) as batch_op:
        batch_op.add_column(sa.Column("end_date", sa.Date(), nullable=True))
        batch_op.add_column(sa.Column("description", sa.Text(), nullable=True))
        batch_op.create_check_constraint(
            "day_off_date_check", "date <= end_date"
        )
    session = Session(bind=op.get_bind())
    session.query(DayOff).update({DayOff.end_date: DayOff.date})
    session.commit()
    with op.batch_alter_table("day_off", schema=None) as batch_op:
        batch_op.alter_column("end_date", nullable=False)

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("day_off", schema=None) as batch_op:
        batch_op.drop_constraint("day_off_date_check", type_="check")
        batch_op.drop_column("description")
        batch_op.drop_column("end_date")

    # ### end Alembic commands ###



import os
import shutil
import math
import cv2
import numpy

from pathlib import Path

from PIL import Image, ImageFile

from zou.app.utils import fs


ImageFile.LOAD_TRUNCATED_IMAGES = True

Image.MAX_IMAGE_PIXELS = 20000 * 20000
RECTANGLE_SIZE = 150, 100
SQUARE_SIZE = 100, 100
PREVIEW_SIZE = 1200, 0
BIG_SQUARE_SIZE = 400, 400
BIG_RECTANGLE_SIZE = 300, 200


def save_file(tmp_folder, instance_id, file_to_save):
    """
    Save file in given folder. The file must only be temporary saved via
    this function.
    """
    extension = "." + file_to_save.filename.split(".")[-1].lower()
    file_name = instance_id + extension.lower()
    file_path = os.path.join(tmp_folder, file_name)
    file_to_save.save(file_path)
    im = Image.open(file_path)
    if im.mode == "CMYK":
        im = im.convert("RGB")
    im.save(file_path, "PNG")
    return file_path


def convert_jpg_to_png(file_source_path):
    """
    Convert .jpg file located at given path into a .png file with same name.
    """
    file_target_path = Path(file_source_path).with_suffix(".png")

    im = Image.open(file_source_path)
    if im.mode == "CMYK":
        im = im.convert("RGB")
    im.save(file_target_path, "PNG")
    fs.rm_file(file_source_path)
    return file_target_path


def get_file_name(instance_id):
    """
    Build thumbnail file name for given id.
    """
    return "%s.png" % instance_id


def get_full_size_from_width(im, width):
    """
    From given width/g
    """
    im_width, im_height = im.size
    ratio = float(im_height) / float(im_width)
    height = int(math.ceil(width * ratio))
    return (width, height)


def make_im_bigger_if_needed(im, size):
    im_width, im_height = im.size
    width, height = size
    if im_width < width and im_height < height:
        im = im.resize(size, Image.Resampling.LANCZOS)
    return im


def fit_to_target_size(im, size, crop=False):
    """
    Make sure that the image is contained in the size given in parameter
    (shorten width and/or height proporitionnally to expected ratio).
    """
    im_width, im_height = im.size
    width, height = size
    w = width
    h = height
    original_ratio = float(im_width) / float(im_height)
    target_ratio = float(width) / float(height)
    if target_ratio != original_ratio:
        w = math.ceil(height * original_ratio)
        if w > width:
            w = width
            h = int(math.ceil(float(width) / original_ratio))
        im = im.resize((w, h), Image.Resampling.LANCZOS)
    return im


def get_dimensions(file_path):
    """
    Return dimensions of given picture (width and height).
    """
    im = Image.open(file_path)
    return im.size


def turn_into_thumbnail(file_path, size=None):
    """
    Turn given picture into a smaller version.
    """
    im = Image.open(file_path)

    if size is not None:
        (width, height) = size
        if height == 0:
            size = get_full_size_from_width(im, width)
    else:
        size = im.size

    im = make_im_bigger_if_needed(im, size)
    im = fit_to_target_size(im, size)

    im.thumbnail(size, Image.Resampling.LANCZOS)
    if im.mode == "CMYK":
        im = im.convert("RGBA")
    final = Image.new("RGBA", size, (0, 0, 0, 0))
    final.paste(
        im, (int((size[0] - im.size[0]) / 2), int((size[1] - im.size[1]) / 2))
    )
    final.save(file_path, "PNG")
    return file_path


def resize(file_path, size, crop=False):
    """
    Resize given picture
    """
    im = Image.open(file_path)
    if crop:
        im = prepare_image_for_thumbnail(im, size)
    im = im.resize(size, Image.Resampling.LANCZOS)
    if im.mode == "CMYK":
        im = im.convert("RGB")
    im.save(file_path, "PNG")
    return file_path


def prepare_image_for_thumbnail(im, size):
    """
    Crop image to avoid deformation while building the target thumbnail.
    """
    im_width, im_height = im.size
    width, height = size
    original_ratio = float(im_width) / float(im_height)
    target_ratio = float(width) / float(height)
    if target_ratio > original_ratio:
        # image is too tall: take some off the top and bottom
        scale_factor = float(target_ratio) / float(original_ratio)
        crop_width = im_width
        crop_height = math.floor(float(im_height) / scale_factor)
        top_cut_line = (im_height - crop_height) / 2
        im = im.crop(
            flat(0, top_cut_line, crop_width, top_cut_line + crop_height)
        )
    else:
        # image is too wide: take some off the sides
        scale_factor = float(original_ratio) / float(target_ratio)
        crop_width = math.ceil(float(im_width) / scale_factor)
        crop_height = im_height
        side_cut_line = int(float(im_width - crop_width) / 2)
        im = im.crop(
            flat(side_cut_line, 0, side_cut_line + crop_width, crop_height)
        )
    return im


def generate_preview_variants(original_path, instance_id):
    """
    Generate three thumbnails for given picture path.

    1. Rectangle thumbnail
    2. Square thumbnail
    3. Big rectangle thumbnail
    """
    file_name = get_file_name(instance_id)
    variants = [
        ("thumbnails", RECTANGLE_SIZE),
        ("thumbnails-square", SQUARE_SIZE),
        ("previews", PREVIEW_SIZE),
    ]

    result = []
    for picture_data in variants:
        (picture_type, size) = picture_data
        folder_path = os.path.dirname(original_path)
        picture_path = os.path.join(
            folder_path, "%s-%s" % (picture_type, file_name)
        )
        shutil.copyfile(original_path, picture_path)
        turn_into_thumbnail(picture_path, size)
        result.append((picture_type, picture_path))
    return result


def url_path(data_type, instance_id):
    """
    Build thumbnail download path for given data type and instance ID.
    """
    data_type = data_type.replace("_", "-")
    return "pictures/thumbnails/%s/%s.png" % (data_type, instance_id)


def flat(*nums):
    """
    Turn into an int tuple an a enumerable of numbers.
    """
    return tuple(int(round(n)) for n in nums)


def turn_hdr_into_thumbnail(original_path, size=BIG_RECTANGLE_SIZE):
    """
    Turn an HDR file into a thumbnail.
    """
    file_target_path = Path(original_path).with_suffix(".png")
    hdr = cv2.imread(original_path, flags=cv2.IMREAD_ANYDEPTH)
    # Simply clamp values to a 0-1 range for tone-mapping
    ldr = numpy.clip(hdr, 0, 1)
    # Color space conversion
    ldr = ldr ** (1 / 2.2)
    # 0-255 remapping for bit-depth conversion
    ldr = 255.0 * ldr
    cv2.imwrite(str(file_target_path), ldr)
    turn_into_thumbnail(file_target_path, size)
    return file_target_path



"""Add plugin.revision

Revision ID: d80f02824047
Revises: e7e633bd6fa2
Create Date: 2025-05-02 16:08:57.078114

"""

from alembic import op
import sqlalchemy as sa
from alembic import op
import sqlalchemy as sa
from zou.migrations.utils.base import BaseMixin
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.orm.session import Session


# revision identifiers, used by Alembic.
revision = "d80f02824047"
down_revision = "e7e633bd6fa2"
branch_labels = None
depends_on = None

base = declarative_base()


class Task(base, BaseMixin):
    """
    Describes a task done by a CG artist on an entity of the CG production.
    The task has a state and assigned to people. It handles notion of time like
    duration, start date and end date.
    """

    __tablename__ = "task"
    name = sa.Column(sa.String(80), nullable=False)
    description = sa.Column(sa.Text())

    priority = sa.Column(sa.Integer, default=0)
    difficulty = sa.Column(sa.Integer, default=3, nullable=False)
    duration = sa.Column(sa.Float, default=0)
    estimation = sa.Column(sa.Float, default=0)
    completion_rate = sa.Column(sa.Integer, default=0)
    retake_count = sa.Column(sa.Integer, default=0)
    sort_order = sa.Column(sa.Integer, default=0)
    start_date = sa.Column(sa.DateTime)
    due_date = sa.Column(sa.DateTime)
    real_start_date = sa.Column(sa.DateTime)
    end_date = sa.Column(sa.DateTime)
    done_date = sa.Column(sa.DateTime)
    last_comment_date = sa.Column(sa.DateTime)
    nb_assets_ready = sa.Column(sa.Integer, default=0)
    data = sa.Column(JSONB)
    nb_drawings = sa.Column(sa.Integer, default=0)


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("plugin", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("revision", sa.String(length=12), nullable=True)
        )

    with op.batch_alter_table("salary_scale", schema=None) as batch_op:
        batch_op.alter_column(
            "position", existing_type=sa.VARCHAR(length=255), nullable=True
        )
        batch_op.alter_column(
            "seniority", existing_type=sa.VARCHAR(length=255), nullable=True
        )

    with op.batch_alter_table("task", schema=None) as batch_op:
        session = Session(bind=op.get_bind())
        session.query(Task).where(Task.difficulty == None).update(
            {
                Task.difficulty: 3,
            }
        )
        session.commit()
        batch_op.alter_column(
            "difficulty",
            existing_type=sa.INTEGER(),
            nullable=False,
            existing_server_default=sa.text("3"),
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("task", schema=None) as batch_op:
        batch_op.alter_column(
            "difficulty",
            existing_type=sa.INTEGER(),
            nullable=True,
            existing_server_default=sa.text("3"),
        )

    with op.batch_alter_table("salary_scale", schema=None) as batch_op:
        batch_op.alter_column(
            "seniority", existing_type=sa.VARCHAR(length=255), nullable=False
        )
        batch_op.alter_column(
            "position", existing_type=sa.VARCHAR(length=255), nullable=False
        )

    with op.batch_alter_table("plugin", schema=None) as batch_op:
        batch_op.drop_column("revision")

    # ### end Alembic commands ###



"""Fix TaskPersonLink

Revision ID: 3d8e68dffeee
Revises: d97f2730bf7b
Create Date: 2025-07-18 11:39:02.795034

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "3d8e68dffeee"
down_revision = "d97f2730bf7b"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("task_person_link", schema=None) as batch_op:
        batch_op.alter_column("task", new_column_name="task_id")
        batch_op.alter_column("person", new_column_name="person_id")
        batch_op.drop_constraint(
            batch_op.f("assignations_person_fkey"), type_="foreignkey"
        )
        batch_op.drop_constraint(
            batch_op.f("assignations_task_fkey"), type_="foreignkey"
        )
        batch_op.create_foreign_key(None, "person", ["person_id"], ["id"])
        batch_op.create_foreign_key(None, "task", ["task_id"], ["id"])

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("task_person_link", schema=None) as batch_op:
        batch_op.alter_column("task_id", new_column_name="task")
        batch_op.alter_column("person_id", new_column_name="person")
        batch_op.drop_constraint(
            "task_person_link_person_id_fkey", type_="foreignkey"
        )
        batch_op.drop_constraint(
            "task_person_link_task_id_fkey", type_="foreignkey"
        )
        batch_op.create_foreign_key(
            batch_op.f("assignations_task_fkey"), "task", ["task"], ["id"]
        )
        batch_op.create_foreign_key(
            batch_op.f("assignations_person_fkey"),
            "person",
            ["person"],
            ["id"],
        )

    # ### end Alembic commands ###
